// ByteMatrix.h: interface for the ByteMatrix class.

//

//////////////////////////////////////////////////////////////////////



#ifndef __BYTEMATRIX_H__

#define __BYTEMATRIX_H__



#include "Utilities.h"



#include <scew.h>



#ifdef __GRIDIO

  #include "gioapi.h"

#endif



#ifdef __GARPWIN

#include <windows.h>

#endif



// ============================================================================

class ByteMatrix

{

public:

  // Constructor

  ByteMatrix();



  // Destructor

  virtual ~ByteMatrix();



  virtual char * type()	{ return "ByteMatrix"; }



  virtual ByteMatrix * clone() = 0;



  virtual void set(long pos, BYTE val) = 0;

  virtual BYTE get(long pos) = 0;



  virtual void rewind() {};



  virtual void open(char * strFilename) = 0;

  virtual void save() {};

  virtual void close() {};



  virtual bool isOpen() { return false; };



  virtual char * toXML();



  static ByteMatrix * fromXML(char * strXML);

  static ByteMatrix * fromXML(scew_element const * );

};



// ============================================================================

class RawByteMatrixInDisk : public ByteMatrix

{

protected:

  FILE * file;

  char strFilename[CHAR_ARRAY_SIZE];

  int intCursor;



public:

  // Constructor

  RawByteMatrixInDisk();



  // Destructor

  virtual ~RawByteMatrixInDisk();



  virtual char * type()	{ return "ByteMatrixInDisk"; }



  virtual ByteMatrix * clone();



  virtual void set(long pos, BYTE val);

  virtual BYTE get(long pos);



  virtual void rewind();



  virtual void create(char * strFilename, long intFileSize);

  virtual void open(char * strFilename);

  virtual void save() {}; // does nothing

  virtual void close();



  virtual char * toXML();



  virtual bool isOpen();

};



// ============================================================================

class RawByteMatrixInMemory : public RawByteMatrixInDisk

{

protected:

  BYTE * array;

  int intSize;

  bool saved;



public:

  // Constructor

  RawByteMatrixInMemory();



  // Destructor

  virtual ~RawByteMatrixInMemory();



  virtual char * type()	{ return "RawByteMatrixInMemory"; }



  virtual void set(long pos, BYTE val);

  virtual BYTE get(long pos);



  virtual void create(char * strFilename, long intFileSize);

  virtual void open(char * strFilename);

  virtual void save();

  virtual void close();



  BYTE * getArrayPtr();

  int size();

};



// ============================================================================

#ifdef __GARPWIN



class RawByteMatrixInMMFile : public ByteMatrix

{

protected:

  HANDLE hFile;

  HANDLE hMMFile;

  char sFilename[CHAR_ARRAY_SIZE];

  BYTE * bArray;

  int iSize;



public:

  // Constructor

  RawByteMatrixInMMFile();



  // Destructor

  virtual ~RawByteMatrixInMMFile();



  virtual char * type()	{ return "RawByteMatrixInMMFile"; }



  virtual ByteMatrix * clone();



  virtual void set(long pos, BYTE val);

  virtual BYTE get(long pos);



  virtual void create(char * strFilename, long intFileSize);

  virtual void open(char * strFilename);

  virtual void close();



  int size();

};



// ============================================================================

class BitmapByteMatrix : public RawByteMatrixInMemory

{

protected:

  int iColumns, iRows;

  int iColorDepth;

  int iPixelWidth;



public:

  // Constructors

  BitmapByteMatrix();



  // Destructor

  virtual ~BitmapByteMatrix();



  virtual char * type()	{ return "BitmapByteMatrix"; }



  virtual void set(long pos, BYTE val);

  virtual BYTE get(long pos);



  void initialize(int iCDepth, int iColumns, int iRows);

};

#endif



// ============================================================================

#ifdef __GRIDIO



class ArcGridByteMatrix : public ByteMatrix

{

protected:

  char strGridName[CHAR_ARRAY_SIZE];

  int iCursor;



  int iType;

  int iChannel;

  int iEvalType;

  int iRows;

  int iCols;

  double dBndBox[4];

  double dCellSize;

  double dMinValue, dMaxValue, dValueCoef;

  CELLTYPE * oRowBuf;



public:

  // Constructor

  ArcGridByteMatrix();



  // Destructor

  virtual ~ArcGridByteMatrix();



  virtual char * type()	{ return "ArcGridByteMatrix"; }



  virtual ByteMatrix * clone();



  void getIndexes(long pos, int& iRowAux, int& iColAux);

  void getCoordinates(long pos, double& dX, double& dY);

  BYTE ArcGridByteMatrix::scale(double dUnscaledValue);

  double ArcGridByteMatrix::unscale(BYTE bytScaledValue);



  virtual void set(long pos, BYTE val);

  virtual BYTE get(long pos);



  virtual void rewind() {};



  void create(char * strFilename, double dCellSize,

          double dXMax, double dYMax, double dXMin, double dYMin,

        double dMinValue, double dMaxValue);



  void open(char * strFilename);

  virtual void save() {};  // does nothing

  virtual void close();



  virtual bool isOpen();

};



#endif //__GRIDIO



// ============================================================================

class BooleanByteMatrix : public ByteMatrix

{

protected:

  // aux array that stores the positions of all presence points

  // this is used by EnvLayerSet presample methods

  long intPresencePointPosition[MAX_PRESENCE_POINTS];

  int intPresencePointCount;



public:

  // Constructor

  BooleanByteMatrix();



  // Destructor

  virtual ~BooleanByteMatrix();



  virtual char * type()	{ return "BooleanByteMatrix"; }



  virtual ByteMatrix * clone();



  virtual void set(long pos, BYTE val);

  virtual BYTE get(long pos);



  virtual void rewind();



  virtual void open(char * strFilename);

  virtual void save();

  virtual void close();



  virtual bool isOpen();



  virtual char * toXML();



  void copyPresenceArray(int& intPoints, long * intPositionArray);



  int getPresencePointCount()

  { return intPresencePointCount; }

};



// ============================================================================

#endif // #ifndef __BYTEMATRIX_H__



