// ColorScheme.h: ColorScheme class declarations.

//

//////////////////////////////////////////////////////////////////////



#ifndef __COLORSCHEME_H__

#define __COLORSCHEME_H__



#include <windows.h>



// ============================================================================

const int csDefaultScheme        = 0;

const int csSpeciesScheme        = 1;

const int csTemperatureScheme    = 2;

const int csPrecipitationScheme  = 3;

const int csSnowfallScheme       = 4;

const int csSolarRadiationScheme = 5;

const int csElevationScheme      = 6;

const int csSlopeScheme          = 7;

const int csVegetationScheme     = 8;

const int csCustomScheme         = 9;



const int NumOfColorSchemes      = 10;



// ============================================================================

class ColorScheme  

{

protected:

	RGBTRIPLE oColors[256];



public:

	ColorScheme();

	

	virtual void generateColors();



	virtual int type()  { return csDefaultScheme; }



	void setMaskColor();

	void copyColor(int iColor, RGBTRIPLE * oColor);

	RGBTRIPLE * getColor(int iColor);

	void putColor(int iColor, RGBTRIPLE * oColor);

};



// ============================================================================

class SpeciesScheme : public ColorScheme

{

public:

	SpeciesScheme() : ColorScheme() { generateColors(); };

	virtual void generateColors();

	virtual int type()  { return csSpeciesScheme; }

};



// ============================================================================

class TemperatureScheme : public ColorScheme

{

public:

	TemperatureScheme() : ColorScheme() { generateColors(); };

	virtual void generateColors();

	virtual int type()  { return csTemperatureScheme; }

};



// ============================================================================

class PrecipitationScheme : public ColorScheme

{

public:

	PrecipitationScheme() : ColorScheme() { generateColors(); };

	virtual void generateColors();

	virtual int type()  { return csPrecipitationScheme; }

};



// ============================================================================

class SnowfallScheme : public ColorScheme

{

public:

	SnowfallScheme() : ColorScheme() { generateColors(); };

	virtual void generateColors();

	virtual int type()  { return csSnowfallScheme; }

};



// ============================================================================

class SolarRadiationScheme : public ColorScheme

{

public:

	SolarRadiationScheme() : ColorScheme() { generateColors(); };

	virtual void generateColors();

	virtual int type()  { return csSolarRadiationScheme; }

};



// ============================================================================

class ElevationScheme : public ColorScheme

{

public:

	ElevationScheme() : ColorScheme() { generateColors(); };

	virtual void generateColors();

	virtual int type()  { return csElevationScheme; }

};



// ============================================================================

class SlopeScheme : public ColorScheme

{

public:

	SlopeScheme() : ColorScheme() { generateColors(); };

	virtual void generateColors();

	virtual int type()  { return csSlopeScheme; }

};



// ============================================================================

class VegetationScheme : public ColorScheme

{

public:

	VegetationScheme() : ColorScheme() { generateColors(); };

	virtual void generateColors();

	virtual int type()  { return csVegetationScheme; }

};



// ============================================================================

class CustomScheme : public ColorScheme

{

public:

	CustomScheme() : ColorScheme() { generateColors(); };

	virtual void generateColors();

	virtual int type()  { return csCustomScheme; }

};



// ============================================================================

// ============================================================================

class ColorSchemes

{

public:

	char sLayerType[NumOfColorSchemes][256];

	ColorScheme * oSchemes[NumOfColorSchemes];



public:

	ColorSchemes();



	ColorScheme * getScheme(char * sLayerType);

};



// ============================================================================

// ============================================================================



#endif