// Rule.cpp: implementation of the Rule class.

//

//////////////////////////////////////////////////////////////////////



#include "Rule.h"

#include "EnvCellSet.h"

#include "EnvLayerSet.h"

#include "Utilities.h"



#include <scew.h>

#include <math.h>



// ==========================================================================

//  Rule implelentation

// ==========================================================================

Rule::Rule()

{

  Gene = NULL;

  intGenes = 0;



  for (int i = 0; i < 10; i++)

    dblPerformance[i] = 0.0;



  bGeneIsActive = NULL;

  iGeneIndex = NULL;

  iActiveGenes = 0;



  blnNeedsEvaluation = true;

  intGens = 0;

  intTrials = 0;

  intScreener = 0;

  intScreen = 0;

  intLength = 0;

  intNumber = 0;

  intConclusion = 0;

  chrOrigin = (char) 0;

  chrPad = (char) 0;



  lId = 0;

  iOrigGen = 0;



  _pXYs = 0.0;

  _no = 0;

  _dA = 0.0;

  _dSig = 0.0;

}



// ==========================================================================

Rule::~Rule()

{

  if (Gene)

    delete[] Gene;

}



// ==========================================================================

Rule * Rule::clone()

{

  int i;



  Rule * newRule = objFactory();



  for (i = 0; i < 10; i++)

    newRule->dblPerformance[i] = dblPerformance[i];



  newRule->intGenes = intGenes;

  newRule->intGens = intGens;

  newRule->blnNeedsEvaluation = blnNeedsEvaluation;

  newRule->intTrials = intTrials;

  newRule->intScreener = intScreener;

  newRule->intScreen = intScreen;

  newRule->intLength = intLength;

  newRule->chrOrigin = chrOrigin;

  newRule->chrPad = chrPad;



  if (newRule->Gene) delete[] newRule->Gene;

  newRule->Gene = new BYTE[newRule->intLength];



  for (i = 0;i < intLength; i++)

    { newRule->Gene[i] = Gene[i]; }



  // parameters related to the algorithm

  newRule->bGeneIsActive = bGeneIsActive;

  newRule->iGeneIndex = iGeneIndex;

  newRule->iActiveGenes = iActiveGenes;



  newRule->lId = lId;

  newRule->iOrigGen = iOrigGen;



  newRule->_pXYs = _pXYs;

  newRule->_no = _no;

  newRule->_dA = _dA;

  newRule->_dSig = _dSig;



  return newRule;

}



// ==========================================================================

void Rule::copy(Rule * fromRule)

{

  int i;



  if (type() != fromRule->type())

    throw GarpException(1, "Cannot copy rules with different types");



  for (i = 0; i < 10; i++)

    dblPerformance[i] = fromRule->dblPerformance[i];



  intGenes			= fromRule->intGenes;

  intGens				= fromRule->intGens;

  blnNeedsEvaluation	= fromRule->blnNeedsEvaluation;

  intTrials			= fromRule->intTrials;

  intScreener			= fromRule->intScreener;

  intScreen			= fromRule->intScreen;

  intLength			= fromRule->intLength;

  chrOrigin			= fromRule->chrOrigin;

  chrPad				= fromRule->chrPad;



  if (Gene) delete[] Gene;

  Gene = new BYTE[intLength];



  for (i = 0; i < intLength; i++)

    { Gene[i] = fromRule->Gene[i]; }



  // parameters related to the algorithm

  bGeneIsActive = fromRule->bGeneIsActive;

  iGeneIndex    = fromRule->iGeneIndex;

  iActiveGenes  = fromRule->iActiveGenes;



  lId = fromRule->lId;

  iOrigGen = fromRule->iOrigGen;



  _pXYs = fromRule->_pXYs;

  _no = fromRule->_no;

  _dA = fromRule->_dA;

  _dSig = fromRule->_dSig;

}



// ==========================================================================

void Rule::initialize(EnvCellSet * objEnvCellSet, const RuleSet * objRuleSet,

            bool * bGeneIsActivePtr, int * iGeneIndexPtr, int iActGenes)

{

  int i, p;



  // parameters related to the algorithm

  bGeneIsActive = bGeneIsActivePtr;

  iGeneIndex = iGeneIndexPtr;

  iActiveGenes = iActGenes;



  if (!iGeneIndex)

    i = 0;



  intTrials = 0;

  blnNeedsEvaluation = true;

  intGens = 0;

  chrOrigin = type();

  intScreen = 1;

  chrPad = ' ';



  // number of genes is equal to the number of values in a cell

  // dont take the mask into account

  intGenes = objEnvCellSet->get(0)->size() - 1;

  intLength = intGenes * 2;



  if (Gene) delete[] Gene;

  Gene = new BYTE[intLength];



  for (i = 1; i < intGenes; i++)

  {

    Gene[i * 2]     = 0;

    Gene[i * 2 + 1] = 255;

  }



  p = GarpUtil::randint(0, objEnvCellSet->count() - 1);

  Gene[0] = objEnvCellSet->get(p)->values[0];

    Gene[1] = 0;



  intConclusion = Gene[0];

}



// ==========================================================================

char * Rule::toString()

{

  int i;

  char typ = (char) 0;

  char * strText = new char[1024];



  // rule type

  typ = this->type();

  strcpy(strText, "");

  sprintf(strText, "%c ", typ);



  // genes

  for (i = 0; i < intLength; i++)

    sprintf(strText, "%s %3d ", strText, Gene[i]);



  // performance

  for (i = 0; i < 10; i++)

    sprintf(strText, "%s %-5.3f ", strText, dblPerformance[i]);



  // other values



  // check string sizes

  if (strlen(strText) > 1024)

    throw GarpException(82, "String size exceeded in Rule::toString()");



  // return result

  return strText;

}



// ==========================================================================

Rule * Rule::fromXML(scew_element const* pXMLRuleNode)

{

  Rule * rule;

  char * strValue, * strAux, *strToken;

  XML_Char * sAux;

  int i, ct;

  BYTE genes[MAX_ENV_LAYERS];



  scew_element const *pXMLGenes, *pXMLPerf;



  // find out what is the rule type

  strValue = GarpUtil::getXMLAttrAsString(pXMLRuleNode, "Type");



  // create a rule of the specified type

  if		(strValue[0] == 'd') rule = new RangeRule;

  else if (strValue[0] == '!') rule = new NegatedRangeRule;

  else if (strValue[0] == 'a') rule = new AtomicRule;

  else if (strValue[0] == 'r') rule = new LogitRule;

  else

    throw GarpException(48, "Unknown rule type");



  // fill in the genes

  pXMLGenes = scew_element_by_name_including_parent(pXMLRuleNode, (XML_Char *) "Genes");



  if (pXMLGenes)

  {

    sAux = (XML_Char *) scew_element_contents(pXMLGenes);

    strAux = (char *) sAux;

    ct = 0;

    strToken = strtok(strAux, " \n\r\t");

    while (strToken)

    {

      // get current gene

      genes[ct++] = (BYTE) atoi(strToken);

      // next token

      strToken = strtok(NULL, " \n\r\t");

    }



    // copy genes from temporary array to rule

    rule->intGenes = ct / 2;

    rule->intLength = (rule->intGenes) * 2;

    rule->iActiveGenes = rule->intGenes;



    if (rule->Gene) delete[] rule->Gene;

    rule->Gene = new BYTE[rule->intLength];



    for (i = 0; i < ct; i++)

      rule->Gene[i] = genes[i];

  }

  else

    throw GarpException(52, "Rule has no gene.");



  // fill in performance values

  pXMLPerf = scew_element_by_name_including_parent(pXMLRuleNode, (XML_Char *) "Performance");



  if (pXMLGenes)

  {

    sAux = (XML_Char *) scew_element_contents(pXMLPerf);

    strAux = (char *) sAux;

    ct = 0;

    strToken = strtok(strAux, " \n\r\t");

    while (strToken)

    {

      // get current gene

      rule->dblPerformance[ct++] = atof(strToken);

      strToken = strtok(NULL, " \n\r\t");

    }

  }

  else

    throw GarpException(52, "Rule has no performance values.");



  return rule;

}



// ==========================================================================

char * Rule::toXML()

{

  /* XML Sample



    <Rule Type="r">

      <Genes> 1 0 64 128 12 15 0 255 </Genes>

      <Performance> 0.0000 1.2500 ... </Performance>

    </Rule>

  */



  int i;

  char strAux[64];

  char * strXML;



  strXML = new char[1024];



  // rule type

  sprintf(strXML, "<Rule Type=\"%c\" Id=\"%d\" OrigGen=\"%d\">\n", this->type(), lId, iOrigGen);



  // genes

  strcat(strXML, "  <Genes>");

  for (i = 0; i < intLength; i++)

  {

    sprintf(strAux, " %3d", Gene[i]);

    strcat(strXML, strAux);

  }

  strcat(strXML, "</Genes>\n");



  // performance

  strcat(strXML, "  <Performance>");

  for (i = 0; i < 10; i++)

  {

    sprintf(strAux, " %-5.3f", dblPerformance[i]);

    strcat(strXML, strAux);

  }



  strcat(strXML, "</Performance>");



  strcat(strXML, "</Rule>");



  // other values



  // check string sizes

  if (strlen(strXML) > 1024)

    throw GarpException(82, "String size exceeded in Rule::toXML()");



  // return result

  return strXML;

}



// ==========================================================================

char * Rule::getConclusionAsText()

{

  switch (Gene[0])

  {

  case PRESENCE:		return "PRESENCE";	break;

  case ABSENCE:		return "ABSENCE";	break;

  case MASK_VALUE:	return "MASKED";	break;

  case MISSING_VALUE: return "MISSING";	break;

  default:			return "DEFAULT";   break;

  }

}



// ==========================================================================

double Rule::getCertainty(EnvCell * cell)

{ return (Gene[0] == cell->values[0]); }



// ==========================================================================

double Rule::getError(BYTE pred, EnvCell * cell)

{ return (double) abs( pred - cell->values[0] ); }



// ==========================================================================

void Rule::mutate(int intTemperature)

{

  BYTE aux1, aux2, aux_g1, aux_g2;

  int rnd1, rnd2;

  int j, k;



    j = GarpUtil::randint(1, iActiveGenes);

  k = 2 * getGeneIndex(j);



  rnd1 = GarpUtil::randint(-intTemperature, intTemperature);

  rnd2 = GarpUtil::randint(-intTemperature, intTemperature);



  if ((k + 1 > intLength) || (k < 2))

    throw "Array read out of bounds";



  aux1 = Gene[k];

  aux2 = Gene[k + 1];



  Gene[k]     = (BYTE) (aux1 - rnd1);

  Gene[k + 1] = (BYTE) (aux2 + rnd2);



  aux_g1 = MIN(Gene[k], Gene[k + 1]);

  aux_g2 = MAX(Gene[k], Gene[k + 1]);



  Gene[k]     = aux_g1;

  Gene[k + 1] = aux_g2;



  if (type() == 'a')

    Gene[k + 1] = Gene[k];



  blnNeedsEvaluation = true;

  intGens = 0;

  chrOrigin = 'm';

}



// ==========================================================================

bool Rule::similar(Rule * objOtherRule)

{

  bool found;

  int j, k;

  int ng_2k, ng_2k_1, bg_2k, bg_2k_1;



  if (type() == objOtherRule->type())

  {

    // check rule value (presence/absence)

    if (Gene[0] != objOtherRule->Gene[0])

      return 0;



    for (j = 1, found = true; (j < iActiveGenes) && (found); j += 1)

    {

      k = getGeneIndex(j);



      if ((k * 2 + 1 > intLength) || (k * 2 < 2))

        throw "Array read out of bounds";



      ng_2k_1 = Gene[k * 2 + 1];

      ng_2k   = Gene[k * 2];



      bg_2k_1 = objOtherRule->Gene[k * 2 + 1];

      bg_2k   = objOtherRule->Gene[k * 2];



      found = !( ((ng_2k_1 - ng_2k) == 255 &&

            (bg_2k_1 - bg_2k) != 255 ) ||

             ((ng_2k_1 - ng_2k) != 255 &&

              (bg_2k_1 - bg_2k) == 255 ) );

    }



    return found;

  }



  return false;

}



// ==========================================================================

char * Rule::getIfStatement()

{ return "IF"; }



// ==========================================================================

char * Rule::getThenStatement()

{ return "THEN"; }



// ==========================================================================

char * Rule::translate(EnvLayerSet * objDataset)

{

  char * strText, * strAux;

  char sSpeciesId[256];



  strText = new char[2048];

  strcpy(strText, "");



  sprintf(strText, "%-4c %-5.2f %-5.2f %-5.2f %-5.2f %-5.3f\n",

    type(),

    dblPerformance[2],

    dblPerformance[5],

    dblPerformance[8],

    dblPerformance[1],

    dblPerformance[9]);



  if (objDataset->getSpecies())

    strcpy(sSpeciesId, objDataset->getSpecies()->getId());

  else

    strcpy(sSpeciesId, "sp");



  strcat(strText, getIfStatement());

  strcat(strText, " \t");

  strcat(strText, strAux = getConjunction(objDataset)); delete strAux;

  strcat(strText, "\t");

  strcat(strText, getThenStatement());

  strcat(strText, " \t");

  strcat(strText, sSpeciesId);

  strcat(strText, " = ");

  strcat(strText, getConclusionAsText());

  strcat(strText, "\n");



  // check string sizes

  if (strlen(strText) > 2048)

    throw GarpException(82, "String size exceeded in Rule::translate()");



  return strText;

}



// ==========================================================================

double Rule::testWithData(EnvCellSet * objTrainSet)

{

  const int MAX_UTILS = 10;

  double Utility[MAX_UTILS];



  EnvCell * cell;

  int n, i, rnd, no=0;

  double prediction, certainty, strength;

  double pXs=0,pYs=0,pXYs=0,pYcXs=0,pXSs=0,pYcs=0;



  for (i = 1; i < MAX_UTILS; i++)

    Utility[i] = 0.0;



  Utility[0] = 1.0;



  n = objTrainSet->size();

  for(i=0; i < n; i++)

    {

    // Get an in range data point

    rnd = GarpUtil::randint(0, n - 1);

    cell = objTrainSet->get(rnd);



    strength = getStrength(cell);

    certainty = getCertainty(cell);

    prediction = getError(128, cell);



    pXs  += strength;

    pYs  += certainty;

    pYcs += prediction;



    if (strength > 0)

    {

      pXSs  += strength;

      pYcXs += getError((BYTE)prediction, cell);

      pXYs  += (MIN(certainty, strength)) / strength;

      no++;

    }

    }



  // Priors

  Utility[1] = pXs/n;		// proportion

  Utility[2] = pYs/n;		// Prior probability

  Utility[3] = pYcs/n;



  // Posteriors

  if (no > 0)

    Utility[4] = pXSs/no;

  else

    Utility[4] = 0;



  if (no > 0)					// Posterior probability

    Utility[5] = pXYs/no;

  else

    Utility[5] = 0;



  if (no>0)

    Utility[6] = (pYcXs/no);

  else

    Utility[6] = 0;



  Utility[7] = ((double)no)/n;



  // Crisp Significance

  if (no >= MIN_SIG_NO && Utility[2] > 0 && Utility[2] < 1.0)

    Utility[8]= (pXYs-Utility[2] * no) / sqrt(no * Utility[2] * (1 - Utility[2]));

  else Utility[8] = 0;



  Utility[9] = 0.0;



  //flags!!! not implemented yet

  //if (Postflag)

    Utility[0] *= Utility[5];



  //if (Compflag)

  //	Utility[0] *= Utility[7];



  //if (Sigflag)

    Utility[0] *= Utility[8];



  //if (Ecoflag)

  //	Utility[0] *= ecoSpace();



  //if (Lengthflag)

  //	Utility[0] /= length();

  //*/



  // Record performance in rule



  for (i = 0; i < 10; i++)

    dblPerformance[i] = Utility[i];



  _pXYs = pXYs;

  _no = no;

  _dA = pYs / n;

  _dSig = Utility[8];



  return (Utility[0]);

}



// ==========================================================================

//  GarpRule implelentation

// ==========================================================================

GarpRule::GarpRule() { }



// ==========================================================================

GarpRule::~GarpRule() { }



// ==========================================================================

//  RangeRule implelentation

// ==========================================================================

RangeRule::RangeRule() { }



// ==========================================================================

RangeRule::~RangeRule() { }



// ==========================================================================

void RangeRule::fromString(char * strRule)

{

}



// ==========================================================================

void RangeRule::initialize(EnvCellSet * objEnvCellSet, const RuleSet * objRuleSet,

               bool * geneIsActivePtr, int * geneIndexPtr, int iActGenes)

{

  int i, j, k;



  // call parent initialize

  GarpRule::initialize(objEnvCellSet, objRuleSet, geneIsActivePtr, geneIndexPtr, iActGenes);



  // loop iterates through variables

  for(k = 1; k < iActiveGenes; k++)

  {

    i = GarpUtil::randint(1, iActiveGenes);



    j = getGeneIndex(i);



    if ((j * 2 + 1 > intLength) || (j * 2 < 2))

      throw "Array out of bounds";



    bioclimRange(objEnvCellSet, Gene[0], GarpUtil::random() * 0.1, j);

  }

}



// ==========================================================================

void RangeRule::bioclimRange(EnvCellSet * objEnvCellSet, BYTE pred, double level, int var)

{

  BioclimHistogram * histogram;

  int sum, n, UL, LL;



  histogram = objEnvCellSet->getBioclimHistogram();



  sum = 0;



    for (n = 0; n < 256; n++)

    {

    sum += histogram->matrix[pred][var][n];

    if (sum>(level * histogram->matrix[pred][0][pred]))

    {

      // printf("%d %d %f",var,sum,level);

      LL = n;

      break;

    }

  }



    sum = 0;



  for (n = 255; n >= 0; n--)

    {

    sum += histogram->matrix[pred][var][n];

    if (sum > (level * histogram->matrix[pred][0][pred]))

    {

      // printf("%d %d %f",var,sum,level);

      UL = n;

      break;

    }

  }



    Gene[var * 2 + 1] = (BYTE) UL;

    Gene[var * 2]     = (BYTE) LL;

}



// ==========================================================================

bool RangeRule::applyToCell(EnvCell * cell)

{

  // visit each of the genes

  for (int i = 1; i < iActiveGenes; i++)

  {

    if (!((Gene[getGeneIndex(i) * 2] == 0) && (Gene[getGeneIndex(i) * 2 + 1] == 255)))

      if (GarpUtil::notBetween(cell->values[getGeneIndex(i)], Gene[getGeneIndex(i) * 2], Gene[getGeneIndex(i) * 2 + 1]))

        return false;

  }



  return true;

}



// ==========================================================================

char * RangeRule::getConjunction(EnvLayerSet * objDataset)

{

  int i, k;

  double max, min;

  int Firsttime = 0;

  EnvLayer * objLayer;

  char * strAux, * strText = new char[4096];

  char strMin[64], strMax[64];



  strcpy(strText, "");



  for (k = 1; k < iActiveGenes; k++)

    {

    // get the index of the kth active gene

    i = getGeneIndex(k);



    // get the layer correspondent to the current gene

    objLayer = objDataset->getLayerByIndex(i - 1);



    if (objLayer == NULL)

      throw GarpException(17, "Internal error: Layer index out of range");



    // test if variable is relevant to the rule

    if ((Gene[i * 2] != 0 || Gene[i * 2 + 1] != 255))

    {

      if (Firsttime && Firsttime % 3 == 0)

        strcat(strText, "\n\t");



      if (Firsttime == 0)

      {

        strcat(strText, strAux = objLayer->getId()); delete strAux;

      }

        else

      {

        strcat(strText, " AND ");

        strcat(strText, strAux = objLayer->getId()); delete strAux;

      }



      /*

      if (intScreen)

        Used[1][i]++;

        else

        Used[0][i]++;

      */



      min = (double) objLayer->unscale(Gene[i * 2]);

      max = (double) objLayer->unscale(Gene[i * 2 + 1]);



      sprintf(strMin, "%-5.2f", min);

      sprintf(strMax, "%-5.2f", max);



      strcat(strText, "=[");

      strcat(strText, strMin);

      strcat(strText, ",");

      strcat(strText, strMax);

      strcat(strText, "]");

      strcat(strText, strAux = objLayer->getValueUnits()); delete strAux;



      Firsttime++;



    }

  }



  // check string sizes

  if (strlen(strText) > 4096)

    throw GarpException(82, "String size exceeded in RangeRule::getConjunction()");



  return strText;

}



// ==========================================================================

double RangeRule::getStrength(EnvCell * cell)

{

  int a, b, c;

  int i, k;



  for (k = 1; k < iActiveGenes; k++)

    {

    // get the index of the kth active gene

    i = getGeneIndex(k);



    if ((i * 2 + 1 > intLength) || (i * 2 < 2))

      throw "Array out of bounds";



    a = Gene[i * 2];

    b = Gene[i * 2 + 1];

    c = cell->values[i];



    if (!GarpUtil::membership(a, b, c))

      return 0;

    }



  return 1.0;

}



// ==========================================================================

//  NegatedRangeRule implelentation

// ==========================================================================

NegatedRangeRule::NegatedRangeRule() {}

// ==========================================================================

NegatedRangeRule::~NegatedRangeRule() {}

// ==========================================================================

void NegatedRangeRule::fromString(char * strRule)

{

}



// ==========================================================================

bool NegatedRangeRule::applyToCell(EnvCell * cell)

{

  int i, j;



  // visit each of the genes

  for (i = 1; i < iActiveGenes; i++)

  {

    j = getGeneIndex(i);



    if ((j * 2 + 1 > intLength) || (j * 2 < 2))

      throw "Array out of bounds";



    if (!((Gene[j * 2] == 0) && (Gene[j * 2 + 1] == 255)))

      if (GarpUtil::notBetween(cell->values[j], Gene[j * 2], Gene[j * 2 + 1]))

        return true;

  }



  return false;

}



// ==========================================================================

char * NegatedRangeRule::getIfStatement()

{ return "IF NOT"; }



// ==========================================================================

double NegatedRangeRule::getStrength(EnvCell * cell)

{

  double strength, neg_strength;



  strength     = RangeRule::getStrength(cell);

  neg_strength = 1 - strength;



  return neg_strength;

}



// ==========================================================================

//  AtomicRule implelentation

// ==========================================================================

AtomicRule::AtomicRule() { }



// ==========================================================================

AtomicRule::~AtomicRule() { }



// ==========================================================================

void AtomicRule::initialize(EnvCellSet * objEnvCellSet, const RuleSet * objRuleSet,

              bool * geneIsActivePtr, int * geneIndexPtr, int iActGenes)

{

  int i, j, k, p;



  // call inherited initialize

  GarpRule::initialize(objEnvCellSet, objRuleSet, geneIsActivePtr, geneIndexPtr, iActGenes);



  p = GarpUtil::randint(0, objEnvCellSet->size());

  EnvCell * objEnvCell = objEnvCellSet->get(p);



    for (i = 1; i < iActiveGenes; i++)

  {

    j = GarpUtil::randint(1, iActiveGenes);

    k = getGeneIndex(j);



    if ((k * 2 + 1 > intLength) || (k * 2 < 2))

      throw "Array out of bounds";



    Gene[2 * k] = Gene[2 * k + 1] = objEnvCell->values[k];

    }

}



// ==========================================================================

void AtomicRule::fromString(char * strRule)

{

}



// ==========================================================================

bool AtomicRule::applyToCell(EnvCell * cell)

{

  int i;



  // visit each of the genes

  for (i = 1; i < iActiveGenes; i++)

  {

    if (!((Gene[getGeneIndex(i) * 2] == 0) && (Gene[getGeneIndex(i) * 2 + 1] == 255)))

      if (!(cell->values[getGeneIndex(i)] == Gene[getGeneIndex(i) * 2]))

        return false;

  }



  return true;

}



// ==========================================================================

double AtomicRule::getStrength(EnvCell * cell)

{

  for (int i = 1; i < iActiveGenes; i++)

  {

    if (!GarpUtil::membership(Gene[getGeneIndex(i) * 2], Gene[getGeneIndex(i) * 2 + 1], cell->values[getGeneIndex(i)]))

      return 0;

  }



  return 1;

}



// ==========================================================================

char * AtomicRule::getConjunction(EnvLayerSet * objDataset)

{

  int i, k;

  int Firsttime = 0;

  EnvLayer * objLayer;

  char * strAux, * strText = new char[4096];

  char strValue[64];



  strcpy(strText, "");



  for (k = 1; k < iActiveGenes; k++)

    {

    // get the kth gene index

    i = getGeneIndex(k);



    // get the layer correspondent to the current gene

    objLayer = objDataset->getLayerByIndex(i - 1);



    if (objLayer == NULL)

      throw GarpException(18, "Internal error: Layer index out of range");



    // test if variable is relevant to the rule

    if ((Gene[i * 2] != 0 || Gene[i * 2 + 1] != 255))

    {

      if (Firsttime && Firsttime % 3 == 0)

        strcat(strText, "\n\t");



      if (Firsttime == 0)

      {

        strcat(strText, strAux = objLayer->getId()); delete strAux;

      }

        else

      {

        strcat(strText, " AND ");

        strcat(strText, strAux = objLayer->getId()); delete strAux;

      }



      /*

      if (intScreen)

        Used[1][i]++;

        else

        Used[0][i]++;

      */



      sprintf(strValue, "=%-5.2f", objLayer->unscale(Gene[i * 2]));

      strcat(strText, strValue);



      Firsttime++;



    }

  }



  // check string sizes

  if (strlen(strText) > 4096)

    throw GarpException(82, "String size exceeded in AtomicRule::getConjunction()");



  return strText;

}



// ==========================================================================

//  LogitRule implelentation

// ==========================================================================

LogitRule::LogitRule()

{

}



// ==========================================================================

LogitRule::~LogitRule()

{

}



// ==========================================================================

void LogitRule::fromString(char * strRule)

{

}



// ==========================================================================

void LogitRule::initialize(EnvCellSet * objEnvCellSet, const RuleSet * objRuleSet,

               bool * geneIsActivePtr, int * geneIndexPtr, int iActGenes)

{

  int i, j, k;

  double constant, coef[2];



  // call inherited initialize

  GarpRule::initialize(objEnvCellSet, objRuleSet, geneIsActivePtr, geneIndexPtr, iActGenes);



  // TO DO: must take into account the variables in use (may use some, or all)

    for (i = 0; i < iActiveGenes; i++)

  {

    k = GarpUtil::randint(1, iActiveGenes);

    j = getGeneIndex(k);



    regression(objEnvCellSet, j, constant, coef[0], coef[1]);



    Gene[1]     = (int) constant;

    Gene[j * 2]   = (int) coef[0];

    Gene[j * 2 + 1] = (int) coef[1];

    }

}



// ==========================================================================

int LogitRule::regression(EnvCellSet * objEnvCellSet, int dep, double& constant, double& coef1, double& coef2)

{

  double a, b, x, y, xi, yi, xiyi, xi2, xb, xx, xxi, xxiyi, xxi2;

  int i, n, pred;

  BYTE * values;



  n = objEnvCellSet->count();

  pred = 0;



  a = b = x = y = xi = yi = xiyi = xi2 = xb = xx = xxi = xxiyi = xxi2 = 0.0;



  for (i = 0; i < n; i++)

  {

    values = objEnvCellSet->get(i)->values;



    y = (double) values[pred];

    x = (double) values[dep];



    xx =  x * x;

    xi += x;

    yi += y;

    xiyi += x * y;

    xi2 += x * x;



    xxi += xx;

    xxiyi += xx * y;

    xxi2 += xx * xx;

  }



  b = (n * xiyi - xi * yi) / (n * xi2 - (xi * xi));

  coef1 = ((b * 255) + 128);



  a = yi / n - b * xi / n;

  constant = a;

  xb = (n * xxiyi - xxi * yi) / (n * xxi2 - (xxi * xxi));

  coef2 = ((xb * 255) + 128);



  //printf("\nb= %f %f %f %f %f %f %d",b,a,xb,coef[0],coef[1],coef[2],n);



  return n;

}



// ==========================================================================

bool LogitRule::applyToCell(EnvCell * cell)

{ return (getStrength(cell) == 1.0); }



// ==========================================================================

double LogitRule::getStrength(EnvCell * cell)

{

  BYTE * Data;

  int	i, k;

  double	Sum, prob, r;



  Sum = 0.0;

  prob = 0.0;



  Data = cell->values;



  for (k = 1; k < iActiveGenes; k++)

  {

    i = getGeneIndex(k);



    if ((i * 2 + 1 > intLength) || (i * 2 < 2))

      throw "Array read out of bounds";



    if (GarpUtil::membership(Gene[i * 2], Gene[i * 2 + 1], 1) != 255)

    {

      r = (double) (Data[i] / 254.0);



      Sum += ((double) (Gene[i * 2]     - 128)) * r;

      Sum += ((double) (Gene[i * 2 + 1] - 128)) * r * r;



      //printf("\ni:%d %d C:%f %d %f", i,Rule->Gene[i*2],v,Data[i],Sum);

    }



    prob = 1.0 / (1.0 + (double) exp(-Sum));



    //printf("\nprob %f",prob);

  }



  if (prob > 0.5)

    return 1.0;

  else

    return 0.0;

}



// ==========================================================================

void LogitRule::mutate(int intTemperature)

{

  int j, k;



    k = GarpUtil::randint(1, iActiveGenes);

  j = 2 * getGeneIndex(k);



  if ((j > intLength) || (j < 0))

    throw "Array read out of bounds";



  Gene[j]     = Gene[j]     + GarpUtil::randint(-intTemperature, intTemperature);

  Gene[j + 1] = Gene[j + 1] + GarpUtil::randint(-intTemperature, intTemperature);



  blnNeedsEvaluation = true;

  intGens = 0;

  chrOrigin = 'm';

}



// ==========================================================================

bool LogitRule::similar(Rule * objRule)

{

  bool found;

  int k;



  LogitRule * objOtherRule = (LogitRule *) objRule;



  if (type() == objOtherRule->type())

  {

    // check rule value (presence/absence)

    if (Gene[0] != objOtherRule->Gene[0])

      return 0;



    for (k = 2, found = true; (k < intLength) && (found); k ++)

    {

      if ((k > intLength) || (k < 2))

        throw "Array read out of bounds";



      found = !( ((abs(Gene[k] - 128) < 10) &&

            (abs(objOtherRule->Gene[k] - 128) > 10)) ||

             ((abs(Gene[k] - 128)>10) &&

              (abs(objOtherRule->Gene[k] - 128)<10)) );

    }



    return found;

  }



  return false;

}



// ==========================================================================

char * LogitRule::getConjunction(EnvLayerSet * objDataset)

{

  int k, i;

  double value;

  int Firsttime = 0;

  EnvLayer * objLayer;

  char * strAux, * strText = new char[4096];

  char strValue[64];



  strcpy(strText, "");



  for (k = 1; k < iActiveGenes; k++)

  {

    // get the kth gene index

    i = getGeneIndex(k);



    // get the layer correspondent to the current gene

    objLayer = objDataset->getLayerByIndex(i - 1);



    if (objLayer == NULL)

      throw GarpException(19, "Internal error: Layer index out of range");



    // test if variable is relevant to the rule

    if ((Gene[i * 2] != 0 || Gene[i * 2 + 1] != 255))

    {

      // get the coeficient value

      value = ((double)((int) Gene[i * 2] - 128)) / 256.0;



      if (Firsttime && Firsttime % 3 == 0)

        strcat(strText, "\n\t");



      if (value > 0)

        strcat(strText, " + ");



      else

        strcat(strText, " - ");



      strcat(strText, strAux = objLayer->getId()); delete strAux;



      //sprintf(strValue, "*%-4.2f", fabs(value));

      sprintf(strValue, "*%-4.4f", fabs(value));

      strcat(strText, strValue);

      strcat(strText, strAux = objLayer->getValueUnits()); delete strAux;



      Firsttime++;



      /*

      if (intScreen)

        Used[1][i]++;

      else

        Used[0][i]++;

      */

    }

  }



  // get the intercept term

  value = ((double)((int) Gene[1] - 128)) / 256.0;



  if (value > 0)

    strcat(strText, " + ");



  else

    strcat(strText, " - ");



  sprintf(strValue, "%-4.4f", fabs(value));

  strcat(strText, strValue);



  // check string sizes

  if (strlen(strText) > 4096)

    throw GarpException(82, "String size exceeded in LogitRule::getConjunction()");



  return strText;

}



// ==========================================================================

