/*
 *
 *  Copyright (C) 2001 Numeric Solutions.  All Rights Reserved. 
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  Further, this software is distributed without any warranty that it is
 *  free of the rightful claim of any third person regarding infringement
 *  or the like.  Any license provided herein, whether implied or
 *  otherwise, applies only to this software file.  Patent licenses, if
 *  any, provided herein do not apply to combinations of this program with
 *  other software, or any other product whatsoever.
 * 
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Contact information: Numeric Solutions support@numericsolutions.com, or:
 * 
 *  http://www.numericsolutions.com
 *
 */
#include <sstream>
#include <ctype.h>
#include <iostream>
#include <stdio.h>
#include <string>
#include <fstream>
#include <vector>
#include <math.h>
#include <assert.h>

using namespace std;

extern std::vector<string> fileVectorizer (char const *file);
extern std::vector<string> tokenizeVector (vector <string>, int);
extern std::vector<string> getUniqueElements (vector <string>);
extern std::vector<string> getWellSurveyVector (vector <string>, string);
extern int surveyVectorMatch (vector <string>, vector <string>);


/******************************************************************************
//takes two survey vectors and determines if they are the same
 * ****************************************************************************/
int surveyVectorMatch (vector <string> surveyVec, vector <string> surveyMatchVec)
{
	int returnVal = 1;		//one if a match 
  for (int i = 0; i < surveyVec.size (); i++) {
		char const *sLine;
		//convert to the C lib for use of 
		//string manipulations
		sLine = surveyVec[i].c_str ();
		char w[20];
		char x[20];
		char y[20];
		char z[20];
		sscanf (sLine, "%s %s %s %s", w, x, y, z);
		int matchFound = 0;	//becomes one if a match found 
		//get the target data 
		for (int ii = 0; ii < surveyMatchVec.size (); ii++) {
	  	char const *sMLine;
	  	//convert to the C lib for use of 
	  	//string manipulations      
	  	sMLine = surveyMatchVec[ii].c_str ();
	  	char mw[20];
	  	char mx[20];
	  	char my[20];
	  	char mz[20];
	  	sscanf (sMLine, "%s %s %s %s", mw, mx, my, mz);

	  	//now compute & compare 
	  	float fx = atof (x);
	  	float fy = atof (y);
	  	float fz = atof (z);
	  	float fmx = atof (mx);
	  	float fmy = atof (my);
	  	float fmz = atof (mz);
	  	float difx = fx - fmx;
	  	float dify = fy - fmy;
	  	float difz = fz - fmz;

	  	//cout<< mx <<" "<<  x << " " << difx  <<endl;
	  	// see  that there is at least one exact match
	  	if (fabs (difx) <= 2 && fabs (dify) <= 2 && fabs (difz) <= 2){
	   	   matchFound = 1;
			}
	  }
		if (matchFound == 0) {
	  	returnVal = 0;
		}
	}
  return returnVal;
}

/******************************************************************************
//function that returns a vector with the survey data for 
// a given well name that must be in the first column and 
// the output from this will be converted into a 3d array
 * ****************************************************************************/
vector <string> getWellSurveyVector (vector <string> vec, string well)
{
  vector < string > out;
  string line2;
  char const *cline2;
  char const *element2;
  for (int i = 0; i < vec.size (); i++)
    {
      char const *sLine;
      //convert to the C lib for use of 
      //string manipulations
      sLine = vec[i].c_str ();
      char w[20];
      sscanf (sLine, "%s", w);
      if (w == well)
	{
	  //cout<< well <<" "<<  w <<endl;
	  out.push_back (sLine);
	}
    }
  return out;
}

/******************************************************************************
//function that takes the vector and returns the unique 
// elements that are in the vector
 * ****************************************************************************/
vector<string> getUniqueElements (vector<string> in)
{
  std::cout << "nsgeodata.cpp > getUniqueElements " << in.size () << std::endl;
  vector < string > out;
  string line;
  for (int i = 0; i < in.size (); i++) {
		line = in[i];
		// if the out vector is empty then add the first 
		if (out.size () == 0) {
	  	out.push_back (line);
		} else {
	  	//see if already in the out vector and if not then put it there
	  	int flag = 0;
	  	for (int ii = 0; ii < out.size (); ii++) {
	      string outLine = out[ii];
	      //if the line is found do nothing
	      if (line == outLine) {
					flag = 1;
					break;
				}
			}
	  	//if the flag is still 0 then add it
	  	if (flag == 0) {
				out.push_back (line);                                 
	    }
		}
	}
  return out;

}
/******************************************************************************
//function that takes a vector of strings and tokenizes each
//row and returns the token described by the integer
 * ****************************************************************************/
vector <string> tokenizeVector (vector <string> in, int col)
{
  vector < string > out;
  string line;
  char const *cline;
  for (int i = 0; i < in.size (); i++) {
  	line = in[i];
		//convert to the C lib for use of 
		//string manipulations
		cline = line.c_str ();
		char element[200];
		sscanf (cline, "%s", element);
		out.push_back (element);
	}
  return out;
}
/******************************************************************************
// function that takes the name of a file and retunrs the 
//contents in a vector of strings
 * ****************************************************************************/
vector <string> fileVectorizer (char const *file)
{
  vector < string > out;
  ifstream in (file);
  string s;
  while (getline (in, s)) {
		out.push_back (s);
	}
  in.close ();
  return out;
}
