/**
 *    '$RCSfile: DBTablesGenerator.java,v $'
 *
 *     '$Author: tao $'
 *       '$Date: 2004/09/15 00:10:31 $'
 *   '$Revision: 1.17 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2004 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */
package org.ecoinformatics.seek.dataquery;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import org.ecoinformatics.seek.datasource.Attribute;
import org.ecoinformatics.seek.datasource.TableEntity;
import org.ecoinformatics.seek.datasource.UnresolvableTypeException;
import org.ecoinformatics.util.Config;
import org.ecoinformatics.util.DebugPrinter;
import org.ecoinformatics.util.Util;



/**
 * The class to generate db tables base table entity
 * @author Jing Tao
 *
 */

public class DBTablesGenerator implements Runnable
{

  // Constant
  private static final String CREATETEXTABLEPATH = "//sqlEngine/SQLDictionary/textTable/createTextTable";
  private static final String CREATETABLEPATH    = "//sqlEngine/SQLDictionary/createTable";
  private static final String IFEXISTSPATH       = "//sqlEngine/SQLDictionary/dropSuffix";
  private static final String SEMICOLONPATH      = "//sqlEngine/SQLDictionary/semicolon";
  private static final String FIELDSPEPATH       = "//sqlEngine/SQLDictionary/textTable/fieldSeperator";
  private static final String SETTABLEPATH       = "//sqlEngine/SQLDictionary/textTable/setTable";
  private static final String SOURCEPATH         = "//sqlEngine/SQLDictionary/textTable/source";
  private static final String IGNOREFIRSTPATH    = "//sqlEngine/SQLDictionary/textTable/ignoreFirst";
  private static final String VECTORDATATYPE     = "vectorDataType";
  private static final String TEXTFILETYPE       = "textFileType";

  public static final String CREATETEXTTABLE = Config.getValue(CREATETEXTABLEPATH);
  public static final String CREATETABLE = Config.getValue(CREATETABLEPATH);
  public static final String DROPTABLE = "DROP TABLE";
  public static final String IFEXISTS = Config.getValue(IFEXISTSPATH);
  public static final String LEFTPARENTH = "(";
  public static final String RIGHTPARENTH = ")";
  public static final String SEMICOLON = Config.getValue(SEMICOLONPATH);
  public static final String SPACE = " ";
  public static final String COMMA = ",";
  public static final String QUOTE = "\"";
  public static final String FIELDSEPATATOR = Config.getValue(FIELDSPEPATH);
  public static final String SELECT = "SELECT";
  public static final String INSERT = "INSERT INTO";
  public static final String DELETE = "DELETE";
  public static final String WHERE = "WHERE";
  public static final String FROM = "FROM";
  public static final String LIKE = "LIKE";
  public static final String VALUES = "VALUES";
  public static final String AND    = "AND";
  public static final String QUESTION = "?";
  public static final String SETTABLE = Config.getValue(SETTABLEPATH);
  public static final String SOURCE = Config.getValue(SOURCEPATH);
  public static final String IGNOREFIRST = Config.getValue(IGNOREFIRSTPATH);
  public static final String STRING = "String";
  public static final String INTEGER = "Integer";
  public static final String LONG = "Long";
  public static final String DOUBLE = "Double";
  public static final String FLOAT = "Float";
  public static final String DATETIME = "Timestamp";
  public static final String BOOLEAN = "Boolean";


  private Vector dbJavaDataTypeList = new Vector();
  private DebugPrinter dbg = DebugPrinter.getInstance();
  private TableEntity tableEntity;
  private Vector[] givenData;
  private String textFileLocation;
  private String type;
  private boolean isDone        = false;
  private boolean successStatus = false;

  /**
   * This constructor is for non-text file table. Before create the consturctor
   * user should run DBTableNameResolver first.
   * @param tableEntity TableEntity  table will generated base on the object
   * @param givenData Vector[]       data will be load to table. This a vector
   *                                 array, each vector is one row of
   *                                 data. The element in vector is a string
   */
  public DBTablesGenerator(TableEntity tableEntity, Vector[] givenData)
  {
    this.tableEntity = tableEntity;
    this.givenData   = givenData;
    this.type        = VECTORDATATYPE;
  }//DBTablesGenerator

  /**
   * This constructor is for text file table. Before create the consturctor
   * user should run DBTableNameResolver first.
   * @param tableEntity TableEntity
   * @param textFileLocation String
   */
  public DBTablesGenerator(TableEntity tableEntity, String textFileLocation)
  {
    this.tableEntity      = tableEntity;
    this.textFileLocation = textFileLocation;
    this.type             = TEXTFILETYPE;
  }//DBTablesGenerator

  /**
   * A thread to call some private method to generate table
   */
  public void run()
  {
    if (type == null)
    {
      successStatus = false;
      isDone        = true;
    }
    else if (type.equals(VECTORDATATYPE))
    {
      // generate vector table
      successStatus = generateDBTableForGivenData();
      isDone        = true;
    }
    else if (type.equals(TEXTFILETYPE))
    {
      // gernate text table
      successStatus = generateDBTextTable();
      isDone       = true;
    }
    else
    {
      successStatus = false;
      isDone        = true;
    }
  }//run

  /**
   * Method to get success status of generating table
   * @return boolean
   */
  public synchronized boolean getSuccessStatus()
  {
    return this.successStatus;
  }

  /**
   * Method to get isDone status of generationg table
   * @return boolean
   */
  public  synchronized boolean getIsDone()
  {
    return this.isDone;
  }
   /*
    * This is for non-text file tables. And will load data into db.
    * @param givenEntityList Hashtable
    * @param givenData Vector[]  This a vector array, each vector is one row of
    * data. The element in vector is a string
    */
   private  synchronized boolean generateDBTableForGivenData()
   {
     boolean success = false;
     if (tableEntity == null)
     {
       dbg.print("The entity is null and couldn't create table for it", 2);
       return success;
     }
     // get table name
    String tableName = null;
    String url       = null;
    DBTableExistenceChecker checker = null;
    try
    {
      checker = new DBTableExistenceChecker();
      url = tableEntity.getURL();
      if (url == null)
      {
        url = tableEntity.getName();
      }
      dbg.print("url is " + url, 2);
      // if the table already existed, we don't need
      //generate again(url is the key)
      if (checker.isURLExisted(url))
      {
         // to do get the table name and set to table entity
        tableName = checker.getTableName(url);
        tableEntity.setDBTableName(tableName);
        dbg.print("Talbe " + tableName + " is existed for url "+ url, 2);
        success = true;
        return success;
      }
      // if doesn't exited, we need to get the table name for TableEntity.
      // so before run this method, we need run DBTableNameResovler first.
      tableName = tableEntity.getDBTableName();
      dbg.print("The table name " + tableName + " will be generated", 2);
      if (tableName == null || tableName.trim().equals(""))
      {
         dbg.print("The DB table name for given TableEntity object is null " +
                   "and couldn't generate table", 2);
         success = false;
         return success;
      }

    }
    catch (Exception ee)
    {
      dbg.print("The error in generate table is " + ee.getMessage(), 2);
      success = false;
      return success;
    }


     try
     {
        // this is for non-text type table
       String generateTalbeSql = generateDDLForOneEntity(CREATETABLE,
                                                   tableName, tableEntity);
       excuteSQLCommand(generateTalbeSql);
       loadDataIntoTable(tableName, tableEntity, givenData);
       success = true;
     }
     catch (Exception e)
     {
       dbg.print("The error in generate table is " + e.getMessage(), 2);
       success = false;
     }
     // if success, we need store tablename and url
     if (success)
     {
       try
       {
         checker.storeTableRecord(tableName, url);
         tableEntity.setDBTableName(tableName);
       }
       catch(Exception ee)
       {
         dbg.print("The error in generate table is " + ee.getMessage(), 2);
         success =false;
       }
     }

     // if not success, we need drop the generate table
     if (!success)
     {
       //drop the table
       String drop = generateDropSqlCommand(tableName);
       try
       {
         excuteSQLCommand(drop);
         // delete the record from system table
         checker.deleteRecord(tableName, url);
       }
       catch(Exception ee)
       {
         dbg.print("The error in generate table is " + ee.getMessage(), 2);
         success = false;
       }

     }
     return success;

   }//generatetable


   /*
   * This is for text file tables. And will load data into db.
   * @param givenEntityList Hashtable
   * @param givenData Vector[]  This a vector array, each vector is one row of
   * data. The element in vector is a string
   */
  private  synchronized boolean generateDBTextTable()
  {
    boolean success = false;
    if (tableEntity == null)
    {
      dbg.print("The entity is null and couldn't create table for it", 2);
      return success;
    }

    // get table name
    String tableName = null;
    DBTableExistenceChecker checker = null;
    String url = null;
    try
    {
      checker = new DBTableExistenceChecker();
      url = tableEntity.getURL();
      if (url == null)
      {
         url = tableEntity.getName();
      }
       dbg.print("The url in entity is " + url, 2);
      // if the table already existed, we don't need
      //generate again(url is the key)
      if (checker.isURLExisted(url))
      {
        // to do get the table name and set to table entity
        tableName = checker.getTableName(url);
        tableEntity.setDBTableName(tableName);
        dbg.print("Talbe " + tableName + " is existed for url "+ url, 2);
        success = true;
        return success;
      }

       // if doesn't exited, we need to get the table name for TableEntity.
       // so before run this method, we need run DBTableNameResovler first.
       tableName = tableEntity.getDBTableName();
       dbg.print("The table name " + tableName + " will be generated", 2);
       if (tableName == null || tableName.trim().equals(""))
       {
         dbg.print("The DB table name for given TableEntity object is null " +
                   "and couldn't generate table", 2);
         success = false;
         return success;
       }


    }
    catch (Exception ee)
    {
      dbg.print("The error in generateDBTable is " + ee.getMessage(), 2);
      success = false;
      return success;
    }

    try
    {

      int numOfHeadLines = tableEntity.getNumHeaderLines();
      // hsql only handle two scenaro no head line or one head line
      boolean ignoreHeadLines = false;
      if (numOfHeadLines == 0)
      {
        ignoreHeadLines = false;
      }
      else if (numOfHeadLines == 1)
      {
        ignoreHeadLines = true;
      }
      else
      {
        dbg.print("HSQL text table only handle one line header" +
                  " and this entity has " + numOfHeadLines +
                  " headlines", 2);
        success = false;
        return success;
      }

      // if this is attribute row oriented, hsql can't handle it
      String orientation = tableEntity.getOrientation();
      if (orientation != null && orientation.equals(TableEntity.ROWMAJOR))
      {
        dbg.print("DB doesn't handle a text table which attribute is" +
                  " row oriented", 2);
        success = false;
        return success;
      }

      // this is for text type table
      String generateTableSql = generateDDLForOneEntity(CREATETEXTTABLE,
                                                        tableName, tableEntity);
      excuteSQLCommand(generateTableSql);

      // bind text source to table
      String delimiterStr = tableEntity.getDelimiter();
      // need to figure out the delimiter str in db. The format is different
      DelimiterResolver resolver = new DelimiterResolver();
      String dbDelimiter = resolver.resolve(delimiterStr);
      String bindTextFileToTalbeSql =
          generateBindTextFileToTableSQL(tableName, textFileLocation,
                                         dbDelimiter, ignoreHeadLines);

      excuteSQLCommand(bindTextFileToTalbeSql);
      success = true;
    }
    catch (Exception sql)
    {
      dbg.print("The error in generateDBTable is " + sql.getMessage(), 2);
      success = false;
    }

    // if success, we need store tablename and url
    if (success)
    {
      try
      {
        checker.storeTableRecord(tableName, url);
        tableEntity.setDBTableName(tableName);
      }
      catch (Exception ee)
      {
        success = false;
      }
    }

    // if not success, we need drop the generate table
    if (!success)
    {
      //drop the table
      String drop = generateDropSqlCommand(tableName);
      try
      {
        excuteSQLCommand(drop);
        // delete the record from system table
        checker.deleteRecord(tableName, url);
      }
      catch (Exception ee)
      {
        success = false;
      }

    }

    return success;

  }//generateTable

  /*
   * Method to generate drop sql command
   */
   private  synchronized String generateDropSqlCommand(String tableName)
   {
     String sql = DROPTABLE + SPACE + tableName + SPACE + IFEXISTS + SEMICOLON;
     return sql;
   }

   /*
    * Create a table base one given DDL
    */
   private  synchronized void excuteSQLCommand(String sql) throws SQLException, ClassNotFoundException
   {
     Connection conn = null;
     Statement st    = null;
     dbg.print("The sql command to run is " + sql, 2);
     try
     {
       conn = Util.getDBConnection();
       st = conn.createStatement();
       st.execute(sql);
     }
     finally
     {
       st.close();
       conn.close();
     }
   }//generateTable

   /*
    * Method to load data into table. If error happend, it will roll back.
    * Vector is String vector in vector array data.
    */
   private  synchronized void loadDataIntoTable(String tableName, TableEntity table,
                                  Vector[] data) throws SQLException,
     ClassNotFoundException, IllegalArgumentException, UnresolvableTypeException
   {
     if (data == null)
     {
       return;
     }

     PreparedStatement pStatement = null;
     Connection conn = Util.getDBConnection();
     conn.setAutoCommit(false);
     try
     {
       String insertCommand = generateInsertCommand(tableName, table);
       pStatement = conn.prepareStatement(insertCommand);
       int length = data.length;
       for (int i=0; i<length; i++)
       {
         Vector row = data[i];
         // insert one row data into table
         int sizeOfRow = row.size();
         for (int j = 0; j < sizeOfRow; j++)
         {
           String dataElement = (String) row.elementAt(j);
           // get data type for the vector which already has the cloumn java
           // type info after parsing attribute in private method parseAttributeList
           String javaType = (String) dbJavaDataTypeList.elementAt(j);
           // this method will binding data into preparedstatement base on
           // java data type
           // The index of pstatement start 1 (Not 0), so it should j+1.
           pStatement = setupPreparedStatmentParameter(j+1, pStatement,
               dataElement, javaType);

         }
         pStatement.execute();
       }
     }
     catch(SQLException sqle)
     {
       conn.rollback();
       pStatement.close();
       conn.close();
       throw sqle;
     }
     catch(IllegalArgumentException le)
     {
       conn.rollback();
       pStatement.close();
       conn.close();
       throw le;
     }
     catch(UnresolvableTypeException ue)
     {
       conn.rollback();
       pStatement.close();
       conn.close();
       throw ue;
     }
     conn.commit();
     pStatement.close();
     conn.close();
   }//loadDataIntoTable

   /*
    * Method for generate sql command to create table
    */
   private  synchronized String generateDDLForOneEntity(String tableType, String tableName,
                                          TableEntity table)
                                  throws SQLException, UnresolvableTypeException
   {
     StringBuffer sql            = new StringBuffer();
     String textFileName   = table.getFileName();
     int    headLineNumber = table.getNumHeaderLines();
     String orientation    = table.getOrientation();
     String delimiter      = table.getDelimiter();
     sql.append(tableType);
     sql.append(SPACE);
     sql.append(tableName);
     sql.append(LEFTPARENTH);
     Attribute[] attributeList = table.getAttributes();
     String attributeSql = parseAttributeList(attributeList);
     sql.append(attributeSql);
     sql.append(RIGHTPARENTH);
     sql.append(SEMICOLON);
     String sqlStr = sql.toString();
     dbg.print("The command to create tables is "+sqlStr, 2);
     return sqlStr;
   }//generateDDLForOneEntity
   /*
    * Add attribute defination in create table command. If one attribute is null
    * or has same error an exception will be throw
    */
   private  synchronized String parseAttributeList(Attribute[] list) throws SQLException,
                                                    UnresolvableTypeException
   {
     StringBuffer attributeSql = new StringBuffer();
     if (list == null || list.length == 0)
     {
       dbg.print("There is no attribute defination in entity", 2);
       throw new SQLException("There is no attribute defination in entity");
     }
     int size = list.length;
     DBDataTypeResolver dataTypeResolver = new DBDataTypeResolver();
     boolean firstAttribute = true;
     for (int i = 0; i < size; i++)
     {
       Attribute attribute = list[i];
       if (attribute == null)
       {
         dbg.print("One attribute defination is null attribute list", 2);
         throw new SQLException("One attribute defination is null attribute list");
       }
       String name = attribute.getName();
       String dataType = attribute.getDataType();
       String dbDataType = dataTypeResolver.resolveDBType(dataType);
       String javaDataType = dataTypeResolver.resolveJavaType(dataType);
       dbJavaDataTypeList.add(javaDataType);
       if (!firstAttribute)
       {
         attributeSql.append(COMMA);
       }
       attributeSql.append(name);
       attributeSql.append(SPACE);
       attributeSql.append(dbDataType);
       firstAttribute = false;

     }//for
     return attributeSql.toString();
   }//parseAttributeList

   /*
    * Generate a sql command to for insert data into talbe. Here we use
    * PreparedStatement.
    */
   private  synchronized String generateInsertCommand(String tableName, TableEntity table)
                                                           throws SQLException
   {
     StringBuffer sql = new StringBuffer();
     sql.append(INSERT);
     sql.append(SPACE);
     sql.append(tableName);
     sql.append(LEFTPARENTH);
     Attribute[] list = table.getAttributes();
     if (list == null || list.length == 0)
     {
       dbg.print("There is no attribute defination in entity", 2);
       throw new SQLException("There is no attribute defination in entity");
     }
     int size = list.length;
     // cloumna name part
     boolean firstAttribute = true;
     for (int i = 0; i< size; i++)
     {
       Attribute attribute = list[i];
       if (attribute == null)
       {
        dbg.print("One attribute defination is null attribute list", 2);
        throw new SQLException("One attribute defination is null attribute list");
       }
       String name = attribute.getName();
       if (!firstAttribute)
       {
         sql.append(COMMA);
       }
       sql.append(name);
       firstAttribute = false;
     }
     sql.append(RIGHTPARENTH);
     sql.append(SPACE);
     sql.append(VALUES);
     sql.append(SPACE);
     sql.append(LEFTPARENTH);
     //value part, use ? replace
     firstAttribute = true;
     for (int i = 0; i <size; i++)
     {
       if (!firstAttribute)
       {
         sql.append(COMMA);
       }
       sql.append(QUESTION);
       firstAttribute = false;
     }
     sql.append(RIGHTPARENTH);
     sql.append(SEMICOLON);
     dbg.print("The insert command is " +sql.toString(), 2);
     return sql.toString();
   }

   /*
    * Method to setup data for prepare statment
    */
   private  synchronized PreparedStatement setupPreparedStatmentParameter(int index,
                  PreparedStatement pStatement,
                  String data, String javaDataType) throws SQLException,
                   UnresolvableTypeException, IllegalArgumentException
   {
     if (pStatement == null )
     {
       return pStatement;
     }

     // get rid of white space
     if (data != null)
     {
       data = data.trim();
     }

     // set default type as string
     if (javaDataType == null )
     {
       pStatement.setString(index, data);
     }
     else
     {

         if ( javaDataType.equals(STRING))
         {
           pStatement.setString(index, data);
         }
         else if ( javaDataType.equals(INTEGER))
         {
           pStatement.setInt(index, (new Integer(data)).intValue());
         }
         else if (javaDataType.equals(DOUBLE))
         {
           pStatement.setDouble(index, (new Double(data)).doubleValue());
         }
         else if (javaDataType.equals(FLOAT))
         {
           pStatement.setFloat(index, (new Float(data)).floatValue());
         }
         else if (javaDataType.equals(BOOLEAN))
         {
           pStatement.setBoolean(index, (new Boolean(data)).booleanValue());
         }
         else if (javaDataType.equals(LONG))
         {
           pStatement.setLong(index, (new Long(data)).longValue());
         }
         else if (javaDataType.equals(DATETIME))
         {
           pStatement.setTimestamp(index, Timestamp.valueOf(data));
         }
         else
         {
             throw new  UnresolvableTypeException("This java type " + javaDataType +
                      " has NOT implement in " +
                      "DBTablesGenerator.setupPreparedStatmentParameter method");
         }
     }
     return pStatement;
   }//setupPreparedStatmentParameter

   private  synchronized String generateBindTextFileToTableSQL(String tableName, String textFilePath,
                                       String delimiter, boolean ignoreFirstLine)
                                                 throws SQLException
   {
     if (textFilePath == null || textFilePath.trim().equals(""))
     {
       dbg.print("No file location specify for this text table", 2);
       throw new SQLException("No file location specify for this text table");
     }
     if (delimiter == null)
     {
       throw new SQLException("No delimiter be specified in metadata");
     }
     StringBuffer sql = new StringBuffer();
     sql.append(SETTABLE);
     sql.append(SPACE);
     sql.append(tableName);
     sql.append(SPACE);
     sql.append(SOURCE);
     sql.append(SPACE);
     sql.append(QUOTE);
     sql.append(textFilePath);
     sql.append(SEMICOLON);
     // delimiter part
     sql.append(FIELDSEPATATOR);
     sql.append(delimiter);

     if (ignoreFirstLine)
     {
       sql.append(SEMICOLON);
       sql.append(IGNOREFIRST);
     }
     sql.append(QUOTE);
     dbg.print("The set source command is " + sql.toString(), 2);
     return sql.toString();
   }



}//DBTablesGenerator
