/**
 *    '$RCSfile: DataCacheManager.java,v $'
 *
 *     '$Author: rspears $'
 *       '$Date: 2004/10/13 18:13:30 $'
 *   '$Revision: 1.9 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2003 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */
package org.ecoinformatics.seek.datasource;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.util.Vector;

import org.ecoinformatics.seek.querybuilder.DBUIUtils;
import org.ecoinformatics.util.*;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * A Singleton that managers DataCaheItems
 */
public class DataCacheManager
{
    // static data member
    private static String           mCacheFileName;
    private static DataCacheManager mGlobalCacheManager = null;
    
    // non-static members
    protected Vector                mCache              = new Vector();
    
    /**
     * Returns the singleton
     * @return
     */
    public static DataCacheManager getInstance()
    {
        if (mGlobalCacheManager == null)
        {
            mCacheFileName      = "";
            mGlobalCacheManager = new DataCacheManager();
        }
        return mGlobalCacheManager;
    }
    
    /**
     * Default Constructor
     */
    protected DataCacheManager()
    {
        mCacheFileName = prepareCacheLocation();
        loadCache();
    }
    
    protected void finalize()
    {
        int x = 0;
    }
    
    /**
     * Returns the length of the cache
     * @return
     */
    public int getSize()
    {
        return mCache.size();
    }
    
    /**
     * Returns all the items in the cache
     * @return
     */
    public Vector getItems()
    {
        return mCache;
    }
    
    /**
     * CReates a new object of the class name
     * @param aClassName name of class to be created
     * @return the object
     */
    protected DataCacheItem getNewItemFromClassName(String aClassName)
    {
        try 
        {
            Class classObj = Class.forName(aClassName);
            return (DataCacheItem)classObj.newInstance();
        } catch (Exception e) 
        {
            System.err.println(e);
        }
        
        return null;
    }
    
    /**
     * Loads cache from disk
     * @return
     */
    public boolean loadCache()
    {
        Document dom = DBUIUtils.readXMLFile2DOM(mCacheFileName);
        if (dom != null) 
        {
            Node cache = DBUIUtils.findNode(dom, "cache");
            if (cache != null)
            {
                NodeList list = cache.getChildNodes();
                if (list != null)
                {
                    for (int i=0;i<list.getLength();i++)
                    {
                        Node child = list.item(i);
                        if (child.getNodeType() != Node.TEXT_NODE)
                        {
                            String className = DBUIUtils.findAttrValue(child, "classname");
                            try 
                            {
                                DataCacheItem item = getNewItemFromClassName(className);
                                if (item != null)
                                {
                                    item.buildFromDOM(child);
                                    //if (item.loadData())
                                    //{
                                        addItem(item);
                                    //}
                                }
                            } catch (Exception e) {}
                        }
                    }
                }
            }
        }
        return false;
    }
    
    /**
     * Creates the directories for storing the cache (if necessary)
     * @return the path to the cache
     */
    protected String prepareCacheLocation()
    {
        if (mCacheFileName.length() == 0)
        {
            StringTokenizer st = new StringTokenizer(Util.DBPATH, "/");
            StringBuffer dirPath = new StringBuffer();
            while (st.hasMoreTokens()) 
            {
                String dirName = st.nextToken();
                if (dirName.length() > 0)
                {
                    if (dirPath.length() > 0) 
                    {
                        dirPath.append("/");
                    }
                    dirPath.append(dirName);
                    File file = new File(dirPath.toString());
                    if (!file.exists())
                    {
                        file.mkdir();
                    }
                }
            }
        }
        return Util.DBPATH + "data_cache.dat";
    }
    
    /**
     * Returns the data path to the cache
     * @return
     */
    public String getDataPath()
    {
        if (mCacheFileName.length() == 0)
        {
            
        }
        return Util.DBPATH;
    }
    
    /**
     * Saves all the items out to disk
     * @return true on success
     */
    public boolean saveCache()
    {
        // This must be done first, it fills in the mLocalFileName
        for (Enumeration e = mCache.elements(); e.hasMoreElements();)
        {
            DataCacheItem item = (DataCacheItem)e.nextElement();
            item.saveData();
        }
        
        try
        {
          BufferedWriter bufWriter = new BufferedWriter(new FileWriter(mCacheFileName));
          bufWriter.write(toXMLString());
          bufWriter.close();
        } catch (Exception e)
        {
            System.err.println("saveCache - " +e);
            return false;
        }
        return true;
    }
    
    /**
     * Adds item to cache vector
     * @param aItem
     */
    protected void addItem(DataCacheItem aItem)
    {
        mCache.addElement(aItem);
    }
    
    /**
     * Removes item from cache
     * @param aItem to be removed
     */
    public void removeItem(DataCacheItem aItem)
    {
        mCache.removeElement(aItem);
    }
    
    /**
     * Retrieves a single item
     * @param aInx the index of the item to be gotten
     */
    public DataCacheItem getItemAt(int aInx)
    {
        if (aInx >= 0 && aInx < mCache.size())
        {
          return (DataCacheItem)mCache.elementAt(aInx);
        }
        return null;
    }
    
    /**
     * Removes a single item
     * @param aInx the index of the item to be removed
     */
    public void removeItem(int aInx)
    {
        if (aInx > 0 && aInx < mCache.size())
        {
          DataCacheItem item = (DataCacheItem)mCache.elementAt(aInx);
          item.clear();
          mCache.removeElementAt(aInx);
        }
    }
    
    /**
     * Clear the cache and removes all items
     *
     */
    public void clear()
    {
        for (Enumeration e = mCache.elements(); e.hasMoreElements();)
        {
            DataCacheItem item = (DataCacheItem)e.nextElement();
            item.clear();
        }
        mCache.clear();
    }
    
    /**
     * Clear the cache and removes all items
     *
     */
    public void clearAllByName(String aName)
    {
        Vector itemsToRemove = new Vector();
        for (Enumeration e = mCache.elements(); e.hasMoreElements();)
        {
            DataCacheItem item = (DataCacheItem)e.nextElement();
            if (item.getName().equals(aName))
            {
                itemsToRemove.addElement(item);
            }
        }
        for (Enumeration e = itemsToRemove.elements(); e.hasMoreElements();)
        {
            DataCacheItem item = (DataCacheItem)e.nextElement();
            item.clear();
            mCache.remove(item);
        }
    }
    
    /**
     * Look up an item in the cache, if it is not there than it creates one
     * @param aName the name
     * @param aResourceName the resource name
     * @return the item
     */
    public DataCacheItem getCacheItem(DataCacheListener aListener,
                                      String aName, 
                                      String aResourceName,
                                      String aClassName)
    {
        return getCacheItem( aListener, aName, aResourceName, aClassName, false);
    }
    
    /**
     * Look up an item in the cache, if it is not there than it creates one
     * @param aName the name
     * @param aResourceName the resource name
     * @return the item
     */
    public DataCacheItem getCacheItem(DataCacheListener aListener,
                                      String aName, 
                                      String aResourceName,
                                      String aClassName,
                                      boolean aIgnoreResOnSearch)
    {
        boolean found = false;
        for (Enumeration e = mCache.elements(); e.hasMoreElements();)
        {
            DataCacheItem item = (DataCacheItem)e.nextElement();
            if (item.getName().equals(aName) && 
                ((item.getResourceName().equals(aResourceName) && !aIgnoreResOnSearch) || aIgnoreResOnSearch) &&
                item.getSerialClassName().equals(aClassName))
            {
                item.removeAllListeners();
                item.addListener(aListener);
                return item;
            }
        }
        
        DataCacheItem item = getNewItemFromClassName(aClassName);
        if (item != null)
        {
            item.setName(aName);
            item.setResourceName(aResourceName);
            item.addListener(aListener);
            mCache.addElement(item);
        }
        
        return item;
    }
    
    /**
     * Creates an XML string that describes the cache (this is what is saved out to disk)
     * @return
     */
    public String toXMLString()
    {
        StringBuffer strBuf = new StringBuffer();
        strBuf.append("<cache>\n");
        for (Enumeration e = mCache.elements(); e.hasMoreElements();)
        {
            DataCacheItem item = (DataCacheItem)e.nextElement();
            strBuf.append(item.toXMLString());
        }
        strBuf.append("</cache>\n");
        return strBuf.toString();
    }
}
