/**
 *    '$RCSfile: EcogridMetaDataCacheItem.java,v $'
 *
 *     '$Author: rspears $'
 *       '$Date: 2004/10/07 19:09:23 $'
 *   '$Revision: 1.3 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2003 The Regents of the University of California.
 * All rights reserved.
 * 
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */
package org.ecoinformatics.seek.datasource;

import java.net.URL;

import org.ecoinformatics.ecogrid.client.EcogridQueryClient;
import org.ecoinformatics.seek.querybuilder.DBUIUtils;
import org.w3c.dom.Node;

/**
 * @author globus
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
public class EcogridMetaDataCacheItem extends DataCacheItem
{
    protected String mEndPoint = null;
    protected String mRecordId = null;
    
    /**
     * 
     *
     */
    public EcogridMetaDataCacheItem()
    {
        super();
    }
    
    /**
     * 
     */
    protected String getSerialClassName()
    {
      return EcogridMetaDataCacheItem.class.getName();
    }
    
    /* (non-Javadoc)
     * @see org.ecoinformatics.seek.datasource.DataCacheItem#doWork()
     */
    public void doWork()
    {
        dbg.print("EcogridMetaDataCacheItem - doing Work mStatus "+mStatus, 2);
        if (mData == null)
        {
            loadData();
        }

        if (!isReady() || mData == null)
        {
            dbg.print("EcogridMetaDataCacheItem URL is: " + mResourceName + "\nEndPoint " + mEndPoint+ "\nmRecordId " + mRecordId, 2);
            
            try {
                // create a ecogrid client object and get the full record from the
                // client
                EcogridQueryClient ecogridClient = new EcogridQueryClient(new URL(mEndPoint));
                if (ecogridClient != null)
                {
                    ecogridClient.createEcoGridQueryLevelOnePortType();
                    mData = ecogridClient.get(mRecordId);
                    mSize = mData.length;
                    dbg.print("EcogridMetaDataCacheItem URL is: complete.", 2);
                    mStatus = CACHE_COMPLETE;
                    saveData();
                    return;
                }
                mStatus = CACHE_ERROR;

            } catch (Exception ee) {
                String error = ee.getMessage();
                System.out.println(error);
                mStatus = CACHE_ERROR;
            }
        }
    }
    
    /**
     * 
     * @return
     */
    protected String getAttrsForXML()
    {
        StringBuffer strBuf = new StringBuffer();
        strBuf.append("   endpoint=\"" + mEndPoint +"\"\n");
        strBuf.append("   recordid=\"" + mRecordId +"\"\n");
        return strBuf.toString();

    }
    
    /**
     * 
     * @param aNode
     */
    public boolean buildFromDOM(Node aNode)
    {
        if (aNode != null) 
        {
            mEndPoint = DBUIUtils.findAttrValue(aNode, "endpoint");
            mRecordId = DBUIUtils.findAttrValue(aNode, "recordid");
        }
        return super.buildFromDOM(aNode);
    }
    
    /**
     * @return Returns the mEndPoint.
     */
    public String getEndPoint()
    {
        return mEndPoint;
    }
    /**
     * @param endPoint The mEndPoint to set.
     */
    public void setEndPoint(String endPoint)
    {
        mEndPoint = endPoint;
    }
    
    /**
     * @return Returns the mRecordId.
     */
    public String getRecordId()
    {
        return mRecordId;
    }
    
    /**
     * @param recordId The mRecordId to set.
     */
    public void setRecordId(String recordId)
    {
        mRecordId = recordId;
    }
}
