/**
 *    '$RCSfile: AnnotationEngine.java,v $'
 *
 *     '$Author: berkley $'
 *       '$Date: 2004/10/15 01:02:24 $'
 *   '$Revision: 1.5 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2004 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

package org.ecoinformatics.seek.sms;

/**
  *  @author Shawn Bowers
  *
  *  This class is the main entry into the semantic
  *  registration/annotation component of the SMS. This class
  *  implements the singleton pattern. Currently, it performs only
  *  basic operations. More comming soon.
  */


import java.net.*;
import java.io.*;
import java.util.*;

import javax.xml.parsers.*;

import com.hp.hpl.jena.ontology.*;
import com.hp.hpl.jena.ontology.tidy.*;
import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.util.iterator.ExtendedIterator;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.reasoner.Reasoner;
import com.hp.hpl.jena.vocabulary.*;

import com.hp.hpl.jena.rdql.*;


import ptolemy.vergil.tree.*;
import ptolemy.moml.*;
import ptolemy.kernel.*;
import ptolemy.kernel.util.*;
import ptolemy.actor.*;

import org.ecoinformatics.seek.util.*;


public class AnnotationEngine {

    private boolean _debug = true; // set to false to suppress debug messages

    private static AnnotationEngine _engine = null; // singleton instance

    private Hashtable _namedObjects = new Hashtable(); // maps lsid's to NamedObjs


    // paths to necessary files
    // TODO: make these more robust ...
    private String LOCAL_PATH = "configs/ptolemy/configs/kepler/";
    private String ONTO_FILE= LOCAL_PATH + "ontology.owl";        
    private String SCHEMA_FILE = LOCAL_PATH + "annotation-schema.owl";   
    private String ANNOTATION_FILE = LOCAL_PATH + "annotations.owl";   

    // constants to hold namespaces
    private String SCHEMA_NS = "http://seek.ecoinformatics.org/annotation-schema#";
    private String ONTO_NS = "http://seek.ecoinformatics.org/ontology#";

    private OntModel _ontModelOntology;	// the ontology model
    private OntModel _ontModelAnnotations; // the annotation model

    



    /**
     * Constructor 
     */
    protected AnnotationEngine() {
	initialize();
    }


    /**
     * @return The unique instance of this class
     * This must be called to create/obtain an instance of the engine.
     */
    public static AnnotationEngine instance() {
	if(_engine == null) {
	    _engine = new AnnotationEngine();
	}
	return _engine;
    }


    /**
     * Adds a new NamedObj instance to the engine
     * @param obj the instance to add, which must have an entityId property
     * @returns true if the item was added successfully
     */
    public boolean addNamedObject(NamedObj obj) {
	List idAtts = obj.attributeList(NamedObjId.class);
	Iterator iter = idAtts.iterator();
	boolean result = iter.hasNext();
	while(iter.hasNext()) {
	    NamedObjId id = (NamedObjId)iter.next();
	    // check if it is already there ... 
	    if(!_namedObjects.containsKey(id.getExpression()))
		_namedObjects.put(id.getExpression(), obj);
	}
	return result;
    }


    /**
     * @returns the named object with the given id string, null if the
     * object doesn't exist
     */
    public NamedObj getNamedObject(String lsid) {
	return (NamedObj)_namedObjects.get(lsid);
    }

    
    /**
     * For testing... dumps out the annotations using n3 syntax.
     */
    public void print() {
	_ontModelAnnotations.write(System.out, "N3");
    }


    /**
     * Reload the ontology and annotation information. Eventually,
     * should check if changes were made prior to loading.
     */ 
    protected void initialize() {
	try { 
	    Checker validate = new Checker(false);
	    debug("loading ontologies and annotations ... ", false);
	    // create and read the ontology model
	    _ontModelOntology = ModelFactory.createOntologyModel(OntModelSpec.OWL_MEM_RDFS_INF, null);
	    _ontModelOntology.read("file:" + ONTO_FILE);
	    // create and read the annotation schema and annotations
	    _ontModelAnnotations = ModelFactory.createOntologyModel(OntModelSpec.OWL_MEM_RDFS_INF, null);
	    _ontModelAnnotations.read("file:" + SCHEMA_FILE);
	    _ontModelAnnotations.read("file:" + ANNOTATION_FILE);
	    debug("OK", true);
	} catch(Exception e) {e.printStackTrace();}
    }


    /**
     * Not Implement. 
     * Finds the active domain of class names of the index
     */
    public Vector getActiveClassNames() {
	// run through the index checking for instances ...
	return new Vector();
    }


    /**
     * Constructs the default actor library that is loaded, e.g., when
     * Kepler is started in graph mode.
     */
    public EntityTreeModel buildDefaultActorLibrary(EntityTreeModel libraryModel) {
	try {
	    // needs to be a hash table!!! (lsid, NamedObj)
	    Hashtable idObjects = new Hashtable();
	    
	    debug("getting ids ... ", false);
	    // iterate through library, getting all NamedObjectIDs
	    getNamedObjIds(libraryModel, libraryModel.getRoot());
	    debug("OK", true);
	    
	    debug("building tree model ... ", false);
	    EntityLibrary root = new EntityLibrary();
	    OntClass thing = _ontModelOntology.getOntClass(OWL.Thing.getURI());
	    Iterator iter = thing.listSubClasses(true); // true means direct subclasses
	    while(iter.hasNext())
		buildTreeModel(root, (OntClass)iter.next());
	    debug("OK", true);
	    
	    return new VisibleTreeModel(root);
	} catch(Exception e) {
	    e.printStackTrace(); 
	}

	return null;
    }


    //
    // recursive helper function to build the new tree model
    //
    private void buildTreeModel(EntityLibrary parent, OntClass currClass) {
	try { 	    
	    EntityLibrary folder = new EntityLibrary();	
	    Workspace workspace = folder.workspace();

	    // check to make sure it isn't some non-ontology class TODO:
	    // change this to hold a list of target namespaces obtained
	    // from loading the ontology
	    if(!currClass.getNameSpace().equals(ONTO_NS))
		return;
	
	    folder.setName(currClass.getLabel(null));
	    folder.setContainer(parent);

	    // get all annotated item instances of the class
	    Iterator iter = getMatchingAnnotatedItemIds(currClass).iterator();

	    // for each id add the item with the same id  (nested loop; ugh!)
	    while(iter.hasNext()) {
		Object id = iter.next();
		NamedObj obj = getNamedObject(id.toString());
		if(obj != null) {
		    // clone into current workspace
		    NamedObj obj2 = (NamedObj)obj.clone(workspace);
		    if(obj2 instanceof ComponentEntity)
			((ComponentEntity)obj2).setContainer(folder);
		    else if(obj2 instanceof Attribute)
			((Attribute)obj2).setContainer(folder);
		}
	    }
	    // get direct subclasses
	    iter = currClass.listSubClasses(true);
	    while(iter.hasNext()) {
		OntClass c = (OntClass)iter.next();
		buildTreeModel(folder, c);
	    }

	} catch(Exception e) {
	    e.printStackTrace();
	}
    }


    //
    // helper function. 
    // returns a list of id strings that are instances of the currClass
    // uses a rdql query.
    // 
    private Vector getMatchingAnnotatedItemIds(OntClass currClass) {
	try {
	    Vector resultIds = new Vector();
	    String str = "";
	    str += "select ?lsid \n";
	    str += "where  (?res, " + "<" + RDF.type + ">, " + "<" + currClass.getURI() + ">), \n";
	    str += "       (?res, " + "<" + SCHEMA_NS + "taggedItem>, ?item), \n";
	    str += "       (?item, " + "<" + SCHEMA_NS + "lsid>, ?lsid)";
	    Query q = new Query(str);
	    q.setSource(_ontModelAnnotations);
	    QueryResults results = new QueryEngine(q).exec();
	    while(results.hasNext()) {
		ResultBinding res = (ResultBinding)results.next() ;
		// Return from get is null if not found
		Object obj = res.get("lsid") ;
		// obj will be a Jena object: resource, property or RDFNode.
		if(obj != null)
		    resultIds.add(obj);
	    }
	    results.close() ;
	    return resultIds;

	} catch(Exception e) {
	    e.printStackTrace();
	    return new Vector();
	}
    }

    
    // 
    // recursive helper function to traverse and pull out the named objects with ids
    //
    private void getNamedObjIds(EntityTreeModel model, Object parent) {
	// check if the parent has an id, and if so add to hashtable
	if(parent instanceof NamedObj)
	    addNamedObject((NamedObj)parent);
	// iterator over children
	for(int i = 0; i < model.getChildCount(parent); i++)
	    getNamedObjIds(model, model.getChild(parent, i));
    }




    // 
    // for testing ...
    //
    private void debug(String str, boolean newline) {
	if(!_debug)
	    return;
	if(newline)
	    System.out.println(str);
	else
	    System.out.print("ANNOTATION ENGINE: " + str);
    }



    /**
     * search
     *
     * default case 
     */
    public Vector search(String classname) {
	return search(classname, true);
    }


    /**
     * @param classname the classname (as a term/keyword) to search for
     * @param approx if true, find approximate term matches, and if
     * false, do exact matches
     *
     * Searches the ontology matching the term and for matches, findes
     * annotated items. For example, search("RichnessIndex") finds all
     * actors that instantiate either the class named "RichnessIndex"
     * or one of its subclasses.
     */
    public Vector search(String classname, boolean approx) {
	// check if valid concept name
	if(!isValidClass(classname, approx)) {
	    System.out.println(">>> didn't find classname");
	    return new Vector();
	}

	Vector result = new Vector(); 
	// find all the matching class names and their subclasses
	Vector classes = getMatchingClassNames(classname, approx);
	Iterator iter = classes.iterator();
	while(iter.hasNext()) {
	    // find the matching lsids for the class name
	    OntClass cls = (OntClass)iter.next();
	    Vector ids = getMatchingAnnotatedItemIds(cls);
	    Iterator idIter = ids.iterator();
	    while(idIter.hasNext()) {
		// get the associated objects
		NamedObj obj = getNamedObject(idIter.next().toString());
		if(obj != null && !result.contains(obj))
		    result.add(obj); // add the results
	    }
	}
	return result;
    }


    // 
    // helper function to find all class names
    //
    private Vector getMatchingClassNames(String classname, boolean approx) {
	Vector initResult = new Vector();
	// get all classes in ontology
	Iterator iter = _ontModelOntology.listClasses(); 
	while(iter.hasNext()) {	
	    // find classes that have a similar name
	    OntClass cls = (OntClass)iter.next();
	    if(approx && approxMatch(cls.getLocalName(), classname))
		initResult.add(cls);
	    else if(!approx && classname.equals(cls.getLocalName()))
		initResult.add(cls);
	}
	Vector result = (Vector)initResult.clone();
	iter = initResult.iterator(); 
	while(iter.hasNext()) {	
	    // find all subclasses of direct classes
	    OntClass cls = (OntClass)iter.next();
	    Iterator clsIter = cls.listSubClasses(false); // direct = false
	    while(clsIter.hasNext()) {
		OntClass subCls = (OntClass)clsIter.next();
		if(!result.contains(subCls)) 
		    result.add(subCls);
	    }
	}
	return result;
    }


    //
    // helper function for search.
    //
    // Note that we are assuming uniformity in that there is only a
    // single namespace for the ontology, and so below, we only check
    // the fragments of the URIs of the classes in the ontology.
    //
    // This operation is just a simple optimization pre-step.
    //
    private boolean isValidClass(String classname, boolean approx) {
	// get the classes in the onto
	Iterator iter = _ontModelOntology.listClasses();
	while(iter.hasNext()) {
	    OntClass c = (OntClass)iter.next();
	    if(!approx && c.getLocalName().equals(classname))
		return true;
	    if(approx && approxMatch(c.getLocalName(), classname))
		return true;
	}
	return false;
     }

    // 
    // helper function for search
    // 
    private boolean approxMatch(String val1, String val2) {
	val1 = val1.toLowerCase();
	val2 = val2.toLowerCase();
	if(val1.indexOf(val2) != -1 || val2.indexOf(val1) != -1)
	    return true;
	return false;
    }


    // 
    // for testing: prints out the annotations.
    //
    public static void main(String [] args) {
	AnnotationEngine eng = AnnotationEngine.instance();
	eng.print();
    }


} //AnnotationEngine