/* A database connection actor...

@Copyright (c) 2002-2003 The Regents of the University of California.
All rights reserved.

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the
above copyright notice and the following two paragraphs appear in all
copies of this software.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

                                                PT_COPYRIGHT_VERSION 2
                                                COPYRIGHTENDKEY
*/

package org.geon;

import ptolemy.actor.TypedAtomicActor;
import ptolemy.actor.TypedIOPort;
import ptolemy.data.Token;
import ptolemy.data.expr.Parameter;
import ptolemy.data.expr.StringParameter;
import ptolemy.data.type.BaseType;
import ptolemy.data.type.Type;
import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.util.*;
import ptolemy.gui.MessageHandler;
import ptolemy.actor.gui.BrowserLauncher;

import ptolemy.data.DBConnectionToken;

import util.*;
//For java APIs
import java.sql.*;
import java.io.*;
import java.util.*;
import java.net.*;
//For XSLT APIs
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.stream.StreamSource;
import javax.xml.transform.stream.StreamResult;

//////////////////////////////////////////////////////////////////////////
//// OpenDBConnection
/**
This actor opens a DB connection and sends a reference to it.
For the mean time the connection is to an MS access DB.

@author Efrat Jaeger
@version $Id: OpenDBConnection.java,v 1.12 2004/10/10 06:22:58 jaeger Exp $
@since Ptolemy II 3.0.2
*/
public class OpenDBConnection extends TypedAtomicActor {

 /** Construct an actor with the given container and name.
 *  @param container The container.
 *  @param name The name of this actor.
 *  @exception IllegalActionException If the actor cannot be contained
 *   by the proposed container.
 *  @exception NameDuplicationException If the container already has an
 *   actor with this name.
 */

public OpenDBConnection(CompositeEntity container, String name)
        throws NameDuplicationException, IllegalActionException  {

    super(container, name);

    dbcon = new TypedIOPort(this, "dbcon", false, true);
    // Set the type constraint.
    dbcon.setTypeEquals(BaseType.DBCONNECTION);

    catalog = new StringAttribute(this, "catalog");
    catalog.setExpression("");
//    _catalog = _none;

    databaseFormat = new StringParameter(this, "database format");
    databaseFormat.setExpression("Oracle");
    databaseFormat.addChoice("Oracle");
    databaseFormat.addChoice("DB2");
    databaseFormat.addChoice("Local MS Access");
    databaseFormat.addChoice("Remote MS Access (using objectweb server)");
    databaseFormat.addChoice("MS SQL Server");
    databaseFormat.addChoice("PostgreSQL");
    databaseFormat.addChoice("MySQL");
    databaseFormat.addChoice("Sybase SQL Anywhere");
    _dbFormat = _ORCL;
    
    driverName = new StringAttribute(this, "driverName");
    databaseURL = new StringAttribute(this, "databaseURL");
    username = new StringParameter(this, "username");
    password = new StringParameter(this, "password");

    _attachText("_iconDescription", "<svg>\n"
            + "<ellipse cx=\"0\" cy=\"-30\" "
            + "rx=\"20\" ry=\"10\"/>\n"
            + "<line x1=\"20\" y1=\"0\" "
            + "x2=\"20\" y2=\"-30\"/>\n"
            + "<line x1=\"-20\" y1=\"0\" "
            + "x2=\"-20\" y2=\"-30\"/>\n"
            + "<line x1=\"-20\" y1=\"0\" "
            + "x2=\"20\" y2=\"0\"/>\n"
            + "</svg>\n");

}

  ///////////////////////////////////////////////////////////////////
  ////                     ports and parameters                  ////

  /** A reference to a db connection
   */

  public TypedIOPort dbcon;

  public StringAttribute catalog;
  public StringParameter databaseFormat;
  public StringAttribute driverName;
  public StringAttribute databaseURL;
  public StringParameter username;
  public StringParameter password;

  public String strFileOrURL;

  ///////////////////////////////////////////////////////////////////
  ////                        public methods                     ////

  /** Callback for changes in attribute values
   *  Load a dbdriver repository, either GEON or jdbc
   *
   *  @param a The attribute that changed.
   *  @exception IllegalActionException If the offsets array is not
   *   nondecreasing and nonnegative.
   */
  public void attributeChanged(Attribute at) throws IllegalActionException {
  	      if (at == databaseFormat) {
              String dbFormat = databaseFormat.stringValue();
              if (dbFormat.equals("Oracle")) {
              	  _driverName = "oracle.jdbc.driver.OracleDriver"; //FIXME: remove .driver.??
                  _dbFormat = _ORCL;
              } else if (dbFormat.equals("DB2")) {
              	  _driverName = "com.ibm.db2.jcc.DB2Driver";
                  _dbFormat = _DB2;
              } else if (dbFormat.equals("Local MS Access")) {
              	  _driverName = "sun.jdbc.odbc.JdbcOdbcDriver";
                  _dbFormat = _LACCS;
              } else if (dbFormat.startsWith("Remote MS Access")) {
              	  _dbFormat = _RACCS;
              } else if (dbFormat.equals("MS SQL Server")) {
              	_driverName = "com.microsoft.jdbc.sqlserver.SQLServerDriver";
                  _dbFormat = _MSSQL;
              } else if (dbFormat.equals("PostgreSQL")) {
              	_driverName = "org.postgresql.Driver";
                  _dbFormat = _PGSQL;
              } else if (dbFormat.equals("MySQL")) {
              	_driverName = "com.mysql.jdbc.Driver";
              	_dbFormat = _MYSQL;
              } else if (dbFormat.equals("Sybase SQL Anywhere")) {
              	// TODO: To be added..
                  _dbFormat = _SYBASE;
              } else {
                  throw new IllegalActionException(this,
                          "No jdbc driver within the system for " + dbFormat);
              }
          } else {
              super.attributeChanged(at);
          }
   }

  /** Connect to the Virginia Igneous Rock DB
   * outputs a reference to the DB connection.
   * DriverName = sun.jdbc.odbc.JdbcOdbcDriver
   * url = jdbc:odbc:VirginiaIgneousRock
   */

    public void fire() throws IllegalActionException {
      try {

//        String _driverName = driverName.getExpression();
//        System.out.println("driverName >> " + _driverName);
//        String _databaseURL = databaseURL.getExpression();
//        System.out.println("databaseURL >> " + _databaseURL);
        String _username = username.stringValue();
//        System.out.println("username >> " + _username);
        String _password = password.stringValue();
//        System.out.println("password >> " + _password);

/*
        // get jdbc driver
        Class.forName("org.objectweb.rmijdbc.Driver").newInstance();

        // get connection
        String url = "jdbc:odbc:IgneousRock";
        String rmiHost = "//klin-pc.sdsc.edu";
        Connection c = DriverManager.getConnection("jdbc:rmi:" + rmiHost + ":38383/" + url, "", "");
        jdbc:rmi://klin-pc.sdsc.edu:38383/jdbc:odbc:IgneousRock
*/
        _setDBURL();
        Class.forName(_driverName).newInstance();

        Connection con = DriverManager.getConnection(_databaseURL,
                                                   _username,
                                                   _password);
        dbcon.broadcast(new DBConnectionToken(con));

      }
      catch (Exception ex) {
        throw new IllegalActionException(this, ex, "fire exception DB connection");
      }
    }

    public boolean postfire() {
      return false;
    }
    
    private void _setDBURL() throws IllegalActionException {
        _databaseURL = databaseURL.getExpression();
        switch(_dbFormat) { 
	        case _ORCL: 
	        	if (!_databaseURL.trim().startsWith("jdbc:oracle:thin:@")) {
	        		if (_databaseURL.trim().startsWith("jdbc:oracle:")) {// a different driver type is spcified.
	        			int ind = _databaseURL.indexOf("@");
	        			if (ind > -1) {
	        				_databaseURL = "jdbc:oracle:thin:@" + _databaseURL.substring(ind);
	        	  	    } else throw new IllegalActionException(this, 
	        	  	    		"Illegal database URL: " + _databaseURL);
	        		} else {
	        			_databaseURL = "jdbc:oracle:thin:@" + _databaseURL;
	        		}            	  	  
	        	}
	            break;
	        case _DB2: 
	        	if (!_databaseURL.trim().startsWith("jdbc:db2:")) {
	        	  	  _databaseURL = "jdbc:db2:" + _databaseURL;
	        	}
	            break;
	        case _LACCS: 
	      	  	if (!_databaseURL.trim().startsWith("jdbc:odbc:")) {
	      	  		_databaseURL = "jdbc:odbc:" + _databaseURL;
	      	  	}
	      	  	break;
	        case _RACCS: 
                //TODO: TAKE CARE OF HOW TO SPECIFY DRIVER FOR REMOTE ACCESS DB.
	        	break;
	        case _MSSQL: 
	        	if (!_databaseURL.trim().startsWith("jdbc:microsoft:sqlserver:")) {
	        		_databaseURL = "jdbc:microsoft:sqlserver:" + _databaseURL;
	        	}
	            break; 
	        case _PGSQL: 
	        	if (!_databaseURL.trim().startsWith("jdbc:postgresql:")) {
	        	  	  _databaseURL = "jdbc:postgresql:" + _databaseURL;
	        	}
	            break; 
	        case _MYSQL: 
	        	if (!_databaseURL.trim().startsWith("jdbc:mysql:")) {
	        		_databaseURL = "jdbc:mysql:" + _databaseURL;
	        	}
	            break; 
	        case _SYBASE: 
	        	// TODO: To be added..
	            break; 
	        default: 
	            System.out.println(databaseFormat.getExpression() + " is not supported"); 
        } 
    }
    
    // An indicator for the db format.
    private int _dbFormat;
    private String _databaseURL = "";
    private String _driverName = "";

    // Constants used for more efficient execution.
    private static final int _ORCL = 0;
    private static final int _DB2 = 1;
    private static final int _LACCS = 2;
    private static final int _RACCS = 3;
    private static final int _MSSQL = 4;
    private static final int _PGSQL = 5;
    private static final int _MYSQL = 6;
    private static final int _SYBASE = 7;

}