/**
 *  '$RCSfile: EcoGridQueryServicesControllerTest.java,v $'
 *  Copyright: 2000 Regents of the University of California and the
 *              National Center for Ecological Analysis and Synthesis
 *    Authors: @authors@
 *    Release: @release@
 *
 *   '$Author: tao $'
 *     '$Date: 2004/10/11 23:44:25 $'
 * '$Revision: 1.2 $'
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package test.org.ecoinformatics.seek.ecogrid;

import edu.ucsb.nceas.configxml.ConfigXML;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestResult;
import junit.framework.TestSuite;

import java.util.Vector;

import org.ecoinformatics.seek.ecogrid.DocumentType;
import org.ecoinformatics.seek.ecogrid.EcoGridQueryServicesController;
import org.ecoinformatics.seek.ecogrid.EcoGridService;
import org.ecoinformatics.util.DebugPrinter;
import org.ecoinformatics.util.Util;
import org.ecoinformatics.seek.ecogrid.MetadataSpecificationInterface;


/**
 * <p>Title: SearchScopeTest</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: </p>
 * @author not attributable
 * @version 1.0
 */

public class EcoGridQueryServicesControllerTest  extends TestCase
{
  private EcoGridQueryServicesController controller = new EcoGridQueryServicesController();
  private DebugPrinter dbg                          = DebugPrinter.getInstance();
  
  public EcoGridQueryServicesControllerTest(String name)
  {
    super(name);
  }


  /**
   * Establish a testing framework by initializing appropriate objects
   */
  public void setUp()
  {
  }

  /**
   * Release any objects after tests are complete
   */
  public void tearDown()
  {
  }

  /**
   * Create a suite of tests to be run together
   */
  public static Test suite()
  {
    TestSuite suite = new TestSuite();
    suite.addTest(new EcoGridQueryServicesControllerTest("initialize"));
    suite.addTest(new EcoGridQueryServicesControllerTest("addServiceTest"));
    suite.addTest(new EcoGridQueryServicesControllerTest("updateServiceTest"));
    suite.addTest(new EcoGridQueryServicesControllerTest("removeServiceTest"));
    return suite;
  }

  /**
   * Run an initial test that always passes to check that the test
   * harness is working.
   */
  public void initialize()
  {
    assertTrue(1 == 1);
  }//initialize
  
  /**
   * Test adding a service into controller
   */
  public void addServiceTest()
  {
    try
    {
      dbg.print("Old service in list before add ", 2);
      controller.print();
      EcoGridService newService = new EcoGridService(); 
      newService.setServiceName("New Query Serive");
      newService.setServiceType(MetadataSpecificationInterface.QUERYSERVIETYPE);
      newService.setEndPoint("http://knb.ecoinformatics.org");
      DocumentType type1 = new DocumentType("eml://ecoinformatics.org/eml-2.0.0", "Ecological Metadata Language 2.0.0");
      DocumentType type2 = new DocumentType("http://digir.net/schema/conceptual/darwin/full/2001/1.0", "Darwin Core 1.0");
      DocumentType[] typeList = new DocumentType[2];
      typeList[0] = type1;
      typeList[1] = type2;
      newService.setDocumentTypeList(typeList);
      controller.addService(newService);
      dbg.print("new service in list after add", 2);
      controller.print();    
    }
    catch(Exception e)
    {
      dbg.print("error in add service test "+e.getMessage(), 2);
      assertTrue(1==2);
    }
  }//addServiceTest
  
  /**
   * A method to test update controller list
   */
  public void updateServiceTest()
  {
    try
    {
      dbg.print("Old service in list before update", 2);
      controller.print();
      EcoGridService newService = new EcoGridService();
      newService.setServiceName("New Query Serive");
      newService.setServiceType(MetadataSpecificationInterface.QUERYSERVIETYPE);
      newService.setEndPoint("http://ecogrid.ecoinformatics.org/ogsa/services/org/ecoinformatics/ecogrid/EcoGridQueryInterfaceLevelOneService");
      DocumentType type2 = new DocumentType(
                        "http://digir.net/schema/conceptual/darwin/full/2001/1.0", 
                        "Darwin Core 1.0");
      DocumentType[] typeList = new DocumentType[1];
      typeList[0] = type2;
      newService.setDocumentTypeList(typeList);
      controller.updateService(newService);
      dbg.print("new service in list after update", 2);
      controller.print();  
    }
    catch(Exception e)
    {
      dbg.print("error in update service test "+e.getMessage(), 2);
      assertTrue(1==2);
    }
  }//updateServiceTest
  
  /**
  * A method to test remove a serive from controller list
  */
 public void removeServiceTest()
 {
   try
   {
     dbg.print("Old service in list before remove ", 2);
     controller.print();
     Vector serviceList = controller.getServicesList();
     EcoGridService service =(EcoGridService)serviceList.elementAt(0); 
     controller.removeService(service);
     dbg.print("new service in list after remove ", 2);
     controller.print();  
   }
   catch(Exception e)
   {
     dbg.print("error in remove service test "+e.getMessage(), 2);
     assertTrue(1==2);
   }
 }//updateServiceTest


}//EcoGridQueryServicesControllerTest<
