/**
 *    '$RCSfile: Domain.java,v $'
 *
 *     '$Author: jones $'
 *       '$Date: 2003/12/18 08:35:25 $'
 *   '$Revision: 1.2 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2003 The Regents of the University of California.
 * All rights reserved.
 * 
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

package org.ecoinformatics.seek.datasource;

import java.util.*;

/**
 * This object represents an Attribute Domain, a set of restrictions
 * on legal values that the attribute can contain
 */
public class Domain
{
  public static final int DOM_NONE = 0;
  public static final int DOM_NUMBERTYPE = 1;
  public static final int DOM_BOUNDED = 2;
  public static final int DOM_ENUMERATED = 3;
  public static final int DOM_ENUMENTITY = 4;
  public static final int DOM_TEXT = 5;
  public static final int DOM_DATETIME = 6;
  public static final int DOM_REFERENCE = 7;

  private static final String typeStrings[] = {
    "None",
    "NumberType",
    "Bounded",
    "Enumerated",
    "EnumEntity",
    "Text",
    "DateTime",
    "Reference"
  };

  int domainType;
  String id;
  String precision;
  Vector info;  // vector of objects that describe the domain

  /**
   * Constructor for Domain.
   */
  public Domain(int dom, Vector info, String id)
  {
    domainType = dom;
    this.id = id;
    this.info = info;
    this.precision = null;
  }

  /**
   * Method getDomainType.
   * @return int
   */
  public int getDomainType()
  {
    return domainType;
  }

  /**
   * Method getDomainInfo.
   * get the ith domain info item,
   * or an empty string if i is greater than the number of items
   * @param i
   * @return String
   */
  public String getDomainInfo(int i)
  {
    if( i < info.size() )
    {
      Object o = info.get(i);
      if( o != null ) {
        return o.toString();
      }
    }
    return "";
  }

  /**
   * Method getDomainInfoCount.
   * get the number of items in the info vector
   * @return int
   */
  public int getDomainInfoCount()
  {
    return info.size();
  }

  /**
   * Method getTypeString.
   * get string representation of the domain type
   * @return String
   */
  public String getTypeString()
  {
    return typeStrings[domainType];
  }

  /**
   * Method getTypeFromString.
   * get domain type number for a string, return -1 if not found
   * @param s
   * @return int - domain type
   */
  public static int getTypeFromString(String s)
  {
    for(int i = 0; i < typeStrings.length; i++ )
    {
      if( typeStrings[i].equals(s) )
      {
        return i;
      }
    }
    return -1;
  }

  /**
   * Returns the precision.
   * @return String
   */
  public String getPrecision()
  {
    return precision;
  }

  /**
   * Sets the precision.
   * @param precision The precision to set
   */
  public void setPrecision(String precision)
  {
    this.precision = precision;
  }

}
