/**
 *    '$RCSfile: EcogridQueryDataCacheItem.java,v $'
 *
 *     '$Author: rspears $'
 *       '$Date: 2004/10/07 19:10:22 $'
 *   '$Revision: 1.1 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2003 The Regents of the University of California.
 * All rights reserved.
 * 
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */
package org.ecoinformatics.seek.datasource;

import java.io.StringReader;
import java.net.URL;

import org.ecoinformatics.ecogrid.EcogridQueryParser;
import org.ecoinformatics.ecogrid.EcogridQueryTransformer;
import org.ecoinformatics.ecogrid.EcogridResultsetParser;
import org.ecoinformatics.ecogrid.EcogridResultsetTransformer;
import org.ecoinformatics.ecogrid.QueryType;
import org.ecoinformatics.ecogrid.ResultsetType;
import org.ecoinformatics.ecogrid.ResultsetType_record;
import org.ecoinformatics.ecogrid.client.EcogridQueryClient;
import org.ecoinformatics.seek.querybuilder.DBUIUtils;
import org.w3c.dom.Node;

import ptolemy.util.StringUtilities;

/**
 * This class is not as generic as the name may indicate. It is designed to get the metadata to determine 
 * where the data is stored and then does a Ecogrid "get" to the data. 
 */
public class EcogridQueryDataCacheItem extends DataCacheItem
{
    private QueryType     _query     = null;
    private ResultsetType _resultset = null;
    
    /**
     * 
     *
     */
    public EcogridQueryDataCacheItem()
    {
        super();
    }
    
    /**
     * 
     */
    protected String getSerialClassName()
    {
      return EcogridQueryDataCacheItem.class.getName();
    }
    
    /* (non-Javadoc)
     * @see org.ecoinformatics.seek.datasource.DataCacheItem#doWork()
     */
    public void doWork()
    {
        dbg.print("EcogridQueryDataCacheItem - doing Work mStatus "+mStatus, 2);
        
        if (mData == null)
        {
            loadData();
        }

        if (!isReady() || mData == null)
        {
            dbg.print("EcogridQueryDataCacheItem URL is: " + mResourceName, 2);
            
            try 
            {
                System.out.println(EcogridQueryTransformer.toXMLString(_query));
                // create a client object
                EcogridQueryClient ecogridClient = new EcogridQueryClient(new URL(mResourceName));
                // create ecogrid query interface porty type
                ecogridClient.createEcoGridQueryLevelOnePortType();
                // query ecogrid nodes
                _resultset = ecogridClient.query(_query);
                if (_resultset != null)
                {
                    ResultsetType_record[] records = _resultset.getRecord();
                    if (records != null)
                    {
                        String rsStr = EcogridResultsetTransformer.toXMLString(_resultset);
                        if (rsStr.length() > 0)
                        {
                            mData   = rsStr.getBytes();
                            mSize   = mData.length;
                            mStatus = CACHE_COMPLETE;
                            this.saveData();
                        } else {
                            mStatus = CACHE_ERROR;
                        }
                    } else
                    {
                        mStatus = CACHE_ERROR;
                    }
                } else
                {
                    mStatus = CACHE_ERROR;
                }
                
            } catch (Exception e) 
            {
                System.err.println(e);
                e.printStackTrace();
                mStatus = CACHE_ERROR;
            }

        }        
    }
    
    /**
     * 
     * @param aNode
     */
    public boolean buildFromDOM(Node aNode)
    {
        if (!super.buildFromDOM(aNode))
        {
            return false;
        }

        if (aNode != null) 
        {
            String queryStr = DBUIUtils.findAttrValue(aNode, "query");
            if (queryStr != null && queryStr.length() > 0)
            {
                try
                {
                    StringReader       strReader = new StringReader(StringUtilities.unescapeForXML(queryStr));
                    EcogridQueryParser parser    = new EcogridQueryParser(strReader);
                    parser.parseXML();
                    _query = parser.getEcogridQuery();
                } catch(Exception e)
                {
                    System.err.println("Error processing DOM ["+mName+"]["+mResourceName+"]");
                    System.err.println("queryStr [\n"+queryStr+"\n]");
                    
                    System.err.println(e);
                    e.printStackTrace();
                    return false;
                }
                
            }
        }
        return true;
    }
    
    /**
     * The string with the extra attrs (query)
     * @return
     */
    protected String getAttrsForXML()
    {
        String str = "";
        if (_query != null)
        {
            str = StringUtilities.escapeForXML(EcogridQueryTransformer.toXMLString(_query));
        }
        return " query=\"" + str +"\"\n";
    }
    
    /**
     * @param aQuery the UqeryType object
     */
    public void setQuery(QueryType aQuery)
    {
        _query = aQuery;
    }
    
    /**
     * Returns the data cached as a resultset object
     * @return
     */
    public ResultsetType getResultset()
    {
        if (isReady() && _resultset == null)
        {
            _resultset = EcogridResultsetParser.parse(new String(mData));
        }
        return _resultset;
    }
}
