/**
 *    '$RCSfile: DarwinCoreMetaDataSpecification.java,v $'
 *
 *     '$Author: rspears $'
 *       '$Date: 2004/10/14 13:31:34 $'
 *   '$Revision: 1.5 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2004 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

package org.ecoinformatics.seek.datasource.darwincore;

import java.io.IOException;
import java.util.Hashtable;
import java.util.Vector;

import org.apache.axis.types.URI;
import org.ecoinformatics.ecogrid.EcogridUtils;
import org.ecoinformatics.ecogrid.EcogridUtilsNamedNodeIterator;
import org.ecoinformatics.ecogrid.QueryType;
import org.ecoinformatics.ecogrid.ResultsetType;
import org.ecoinformatics.ecogrid.ResultsetType_resultsetMetadata;
import org.ecoinformatics.ecogrid.ResultsetType_resultsetMetadata_system;
import org.ecoinformatics.ecogrid.digir.impl.DigirProxyImpl;
import org.ecoinformatics.seek.datasource.DataCacheManager;
import org.ecoinformatics.seek.datasource.EcogridDataCacheItem;
import org.ecoinformatics.seek.ecogrid.MetadataSpecificationInterface;
import org.ecoinformatics.seek.ecogrid.SearchQueryGenerator;
import org.ecoinformatics.seek.ecogrid.exception.InvalidEcogridQueryException;
import org.ecoinformatics.util.DebugPrinter;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.util.IllegalActionException;
import ptolemy.kernel.util.NameDuplicationException;

/**
 *     This class is a plugin for DarwinCore namespace to transfer
 *     query group to ecogrid query type. It also provide a method to create
 *     quick search query
 *
 * @author Rod Spears (adpated from EML2MetadataSpecification.java)
 */

public class DarwinCoreMetaDataSpecification implements MetadataSpecificationInterface
{
  private DebugPrinter dbg = DebugPrinter.getInstance();
  private static final String REPLACE      = "#value#";
  private static final String QUERYID      = "darwincore-quick-search-query";

  private String _searchString = "";

  /**
   * Default constructor
   */
  public DarwinCoreMetaDataSpecification()
  {

  } // DarwinCoreMetaDataSpecification

  /**
   * Method to create a quick query search
   * @param value String
   * @return QueryType
   */
  public QueryType getQuickSearchEcoGridQuery(String aValue)
                                         throws InvalidEcogridQueryException
  {
      _searchString = aValue;
      
      Hashtable replaceMapping = new Hashtable();
      replaceMapping.put(REPLACE, aValue);
      SearchQueryGenerator darwinCoreQueryGenerator = new SearchQueryGenerator(QUERYID, replaceMapping);
      QueryType ecogridQuery = darwinCoreQueryGenerator.getQuery();
      return ecogridQuery;

  } // getQuickSearchEcoGridQuery
  
  /**
   * Method to create a ecogrid query
   * @return QueryType
   */
  public QueryType getEcoGridQuery()
  {
    QueryType ecogridQuery = null;
    return ecogridQuery;
  }  // getEcoGridQuery

    /**
     * Returns the human readable Provider and Resource name for a given provider URL and Reource Code
     * @param aURI the provider URL
     * @param aResourceCode the resource code
     * @return the full human readable name
     */
    protected String getDigirResourceName(URI aURI)
    {
        if (aURI == null) return null;
        
        boolean dataWasOK     = false;
        String  dataCacheName = null;
        String  resourceName  = null;
        
        try 
        {
            String  urlStr       = aURI.getScheme() +"://" + aURI.getHost() + (aURI.getPort() > 0 ? ":" + aURI.getPort() : "") + aURI.getPath();
            String str = aURI.toString();
            int inx = str.indexOf('=');
            if (inx == -1) 
            {
                System.out.println("URI is missing the resource: " + str);
                return null;
            }
            
            
            String resourceCode = str.substring(inx+1, str.length()).replaceAll("%20", " ");
        
            DigirProxyImpl digir = new DigirProxyImpl();
            
            String resultStr = "";
            boolean useCache = true;
            if (!useCache) 
            {
                int status = digir.doHTTPRequest(urlStr);
                 resultStr = digir.getDigirResultsetStr();
            } else 
            {
                dataCacheName = "ResourceInfo " + resourceCode;
                EcogridDataCacheItem dataCacheItem = (EcogridDataCacheItem)DataCacheManager.getInstance().getCacheItem(null, dataCacheName, urlStr, EcogridDataCacheItem.class.getName());
                dataCacheItem.doWork();
                if (dataCacheItem.isReady()) {
                    resultStr = new String(dataCacheItem.getData());
                    
                } else
                {
                    dataWasOK = false;
                }
            }
            
            if (resultStr != null)
            {
              Document dom = EcogridUtils.convertXMLStr2DOM(resultStr);
              if (dom != null) 
              {
                  Node name = EcogridUtils.findNodeWithXPath(dom, "/response/content/metadata/provider/name");
                  if (name != null)
                  {
                      String providerName = EcogridUtils.getNodeValue(name);
                      Node provider = EcogridUtils.findNodeWithXPath(dom, "/response/content/metadata/provider");
                      if (provider != null)
                      {
                          EcogridUtilsNamedNodeIterator nodeIter = new EcogridUtilsNamedNodeIterator(provider, "resource"); 
                          while (nodeIter.hasMoreNodes())
                          {
                              Node resNode = nodeIter.nextNode();
                              String resCode = EcogridUtils.findNodeValue(resNode, "code");
                              if (resCode != null && resCode.equals(resourceCode)) 
                              {   
                                  String fullName = providerName + " - " + EcogridUtils.findNodeValue(resNode, "name");
                                  resourceName = fullName.replace('.', ' ');
                                  dataWasOK = true;
                              }
                          }
                      }
                      
                  }
              }
          } else
          {
              System.err.println("Could not contact URL for provide/resource name: " + aURI);
          }
        } catch (Exception e)
        {
            System.err.println(e);
            e.printStackTrace();
            
        } catch (java.lang.NoClassDefFoundError e) {
            System.err.println("NoClassDefFoundError in getting resource name "+aURI);
            System.err.println(e);
            e.printStackTrace();
        }
                                                                                                     
        
        if (dataWasOK) 
        {
            DataCacheManager.getInstance().saveCache(); 
        } else
        {
            DataCacheManager.getInstance().clearAllByName(dataCacheName);            
        }
        return resourceName;
    }
    
    /**
     * Creates a DarwinCore DataSource which extends Source
     * @param aResults
     * @param aEndPointURLStr
     * @param aContainer
     * @param aResultList
     * @throws SAXException
     * @throws IOException
     * @throws EcoGridException
     * @throws NameDuplicationException
     * @throws IllegalActionException
     */
    public boolean addResultsetRecordsToContainer(ResultsetType   aResults,
                                                  String          aEndPointURLStr,
                                                  CompositeEntity aContainer, 
                                                  Vector          aResultList) 
        throws NameDuplicationException, IllegalActionException
      {
          if (aResults == null)
          {
              return false;
          }
          
          int numAdded = 0;
          ResultsetType_resultsetMetadata metaData = aResults.getResultsetMetadata();
          if (metaData != null)
          {
              ResultsetType_resultsetMetadata_system[] system = metaData.getSystem();
              if (system != null)
              {
                  String providerURI = ""; 
                  for (int i=0;i<system.length;i++) 
                  {
                      String fullName = getDigirResourceName(system[i].getValue());
                      if (fullName != null)
                      {
                        DarwinCoreDataSource darwinDS = new DarwinCoreDataSource(aContainer, fullName); 
                        darwinDS.setSearchData(_searchString);
                        darwinDS.setEndpoint(aEndPointURLStr);
                        darwinDS.setProvider(system[i].getValue().toString());
                        aResultList.addElement(darwinDS);
                        numAdded++;
                      } else
                      {
                          System.err.println("Couldn't get full name for provider and resource - Id:[" + system[i].getId() + "] value: [" + system[i].getValue().toString()+"]");
                      }
                  }
              }
          }
          return numAdded > 0;
      }

    /**
     * 
     * @return Returns a unique name that descrobes this class, 
     * often it is the name of the class that implments the interface
     */
    public String getName()
    {
        return getClass().getName();
    }

    /**
     * 
     */
    public String getBriefName()
    {
        return "Museum";
    }
  } // DarwinCoreMetaDataSpecification
