/**
 *    '$RCSfile: QuickSearchAction.java,v $'
 *
 *     '$Author: rspears $'
 *       '$Date: 2004/10/13 20:36:36 $'
 *   '$Revision: 1.16 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2004 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */
package org.ecoinformatics.seek.ecogrid;

import java.awt.event.ActionEvent;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import javax.swing.AbstractAction;

import org.ecoinformatics.ecogrid.QueryType;
import org.ecoinformatics.ecogrid.ResultsetType;
import org.ecoinformatics.seek.ecogrid.exception.InvalidEcogridQueryException;
import org.ecoinformatics.util.DebugPrinter;

import ptolemy.kernel.CompositeEntity;
import ptolemy.vergil.tree.EntityTreeModel;
import ptolemy.vergil.tree.VisibleTreeModel;
/**
 * This class will handle quick search action
 */

public class QuickSearchAction extends AbstractAction
{
  // attributes
  private String          searchValue             = null;
  private String          searchType              = "All";     // Defaults to "All"
  private Vector          searchServicesVector    = null;
  private CompositeEntity resultRoot              = null;
  private DebugPrinter    dbg                     = null;
  // This vector will store a list of vector. One sub-vector is a list of
  //resultrecord got from one search scope after excuting query (probably search
  // mutiple endpoints)
  private Vector          recordVector            = null;
  private DatasetPanel    datasetPanel            = null;
  private EntityTreeModel treeModel               = null;
  
  private Vector          actionList              = new Vector();

  /**
   * Construct of this class
   * @param searchScopeVec the search info includes endspoints, namespace etc
   * @param resultRoot  the root to construct result panel
   * @param name  the name of the action
   * @param datasetPane  the datasetpane to fire the action
   */
  public QuickSearchAction(Vector searchServicesVector,
                           String name, DatasetPanel datasetPane)
  {
    super(name);
    dbg = DebugPrinter.getInstance();
    this.searchServicesVector = searchServicesVector;
    recordVector           = new Vector();
    this.datasetPanel      = datasetPane;

  }

  /**
   * Method to get the recordVector which store the ResultRecord after searching
   * @return Vector
   */
  public Vector getRecordVector()
  {
    return recordVector;
  }

  /**
   * Set up search scope vector
   * @param searchVector Vector
   */
  public void setSearchSerivcesVector(Vector searchVector)
  {
    searchServicesVector = searchVector;
  }


  /**
   * The todo Implementation of abstract method. It will search ecogrid site
   *
   * @param e ActionEvent
   */
  public void actionPerformed(ActionEvent e)
  {
     if (datasetPanel != null)
     {
       searchValue = datasetPanel.getSearchTextFieldValue();
       searchType  = datasetPanel.getSearchDataSrcType();
       resultRoot  = datasetPanel.getResultRoot();
     }
     
     //transfer endpoint based EcoGridService to namespace based Search Scope
     Vector searchScopeVector = transformEcoGridServiceToSearchScope();
     if (searchScopeVector != null && !searchScopeVector.isEmpty() && resultRoot != null
         && searchValue != null && !searchValue.trim().equals(""))
     {
       resultRoot.removeAllEntities();
       // go through every namespace in search scope
       for (int i = 0; i < searchScopeVector.size(); i++)
       {
         // vecotr to store the ResultRecord for one search scope
         Vector resultFromOneScope = new Vector();
         SearchScope searchScope = (SearchScope)searchScopeVector.elementAt(i);
         //String namespace = searchScope.getNamespace();
         // get quick search query from metadata specification class
         MetadataSpecificationInterface metadataSpecClass = searchScope.getMetadataSpecification();
         
         // *** Temporary Code
         String name = metadataSpecClass.getBriefName();
         if (!searchType.equals("All") && !searchType.equals(name))
         {
             continue;
         }
               
             
         QueryType quickSearchQuery = null;
         try
         {
           quickSearchQuery = metadataSpecClass.getQuickSearchEcoGridQuery(searchValue);
         }
         catch(InvalidEcogridQueryException inE)
         {
           dbg.print("The error to generate quick search query " +
                     inE.getMessage(), 2);
           return;
         }
         Vector searchEndPoints = searchScope.getEndPoints();
         if ( searchEndPoints == null )
         {
           dbg.print("No search end points can be found", 2);
           return;
         }
         // go through the end points vector
         searchEndPointsVector(resultFromOneScope, searchEndPoints,
                               quickSearchQuery, metadataSpecClass);
       }//for
     }//if
  }//actionPerformed

  /*cd
   * Go through the search end vector
   */
  private void searchEndPointsVector(Vector    resultFromOneScope,
                                     Vector    searchEndPoints, 
                                     QueryType quickSearchQuery,
                                     MetadataSpecificationInterface metadataSpecClass)
  {

    // go through each search end points and get result set
    for (int j = 0; j < searchEndPoints.size(); j++)
    {

      String url = (String) searchEndPoints.elementAt(j);
      QueryAction doQuery = new QueryAction(this, quickSearchQuery, url, searchValue, 
                                            datasetPanel, metadataSpecClass);
      actionList.addElement(doQuery);
      doQuery.actionPerformed(null);      
    }//for

  }

  /*
   * Method transfer EcoGridService(base on endpoint) vector to SearchScope
   * (base on namespace) vector
   */
  public Vector transformEcoGridServiceToSearchScope()
  {
    Vector searchScopeVector = new Vector();
    if (searchServicesVector  != null)
    {
      int size = searchServicesVector.size();
      //this hashtable will stored the namespace and its searchscope
      Hashtable namespaceAndSearhScopeMap = new Hashtable();
      for (int i=0; i<size; i++)
      {
        EcoGridService service = (EcoGridService)searchServicesVector.elementAt(i);
        if (service!= null)
        {
          String endPoint = service.getEndPoint();
          DocumentType[] typeList = service.getDocumentTypeList();
          if (endPoint != null && !endPoint.trim().equals("") && typeList != null)
          {
             //go through every type
             for (int j=0; j<typeList.length; j++)
             {
               DocumentType type = typeList[j];
               String namespaceStr = type.getNamespace();
               String metadataClassName = type.getMetadataSpecificationClassName();
               if (namespaceStr != null && !namespaceStr.trim().equals("") && 
                   metadataClassName !=null && !metadataClassName.trim().equals(""))
               {
                  if(!namespaceAndSearhScopeMap.containsKey(namespaceStr))
                  {
                    //if this is first namespace we found, create a new searchscope
                    Vector endPointsVector = new Vector();
                    endPointsVector.add(endPoint);
                    try
                    {
                      SearchScope newScope = new SearchScope(namespaceStr, metadataClassName, endPointsVector);
                      namespaceAndSearhScopeMap.put(namespaceStr, newScope);
                      dbg.print("add "  + namespaceStr + " " +
                                metadataClassName + " "+ endPoint + 
                                " into SearchScope", 2);
                    }
                    catch(Exception e)
                    {
                      dbg.print("error create search scope " +e.getMessage(), 2);
                      continue;
                    }
                  }
                  else
                  {
                    //if we already has a SearchScope for this namespace,
                    // add this endpoints to endpoints vector
                    SearchScope existedScope = (SearchScope)namespaceAndSearhScopeMap.get(namespaceStr);
                    dbg.print("Add endpoint "+endPoint + " to a existed searchscope object", 2);
                    existedScope.addSearchEndPoint(endPoint);
                    //overwrite the old value
                    namespaceAndSearhScopeMap.put(namespaceStr, existedScope);
                  }
               }//if
             }//for
          }//if
        }//if
      }//for
      
      //transfer the hashtable to a vector
      Enumeration en = namespaceAndSearhScopeMap.keys();
      while (en.hasMoreElements())
      {
        String key = (String)en.nextElement();
        SearchScope scope = (SearchScope) namespaceAndSearhScopeMap.get(key);
        if (scope!=null)
        {
          searchScopeVector.add(scope);
        }
      }
      
    }//if
    
    return searchScopeVector;
  }
  
  public void resultsetComplete(QueryAction aQA)
  {
      // parse the resultset into and stored it into a vector
      Vector resultVector  = new Vector();
      try
      {
          MetadataSpecificationInterface mdi           = aQA.getMetaDataIFace();
          ResultsetType                  results       = aQA.getResultSet();
          String                         url           = aQA.getURL();    
          mdi.addResultsetRecordsToContainer(results, url, resultRoot, resultVector);
      }
      catch (Exception ee)
      {
        dbg.print("The error to adding resultset to container "+
                    ee.getMessage(), 2);
      }
          
      actionList.removeElement(aQA);
      
      if (resultVector.size() > 0)
      {
          recordVector.add(resultVector);
      }
      
      if (actionList.size() == 0) {
          // sort the resultvector from one scope and add it to the recordVector
          //Collections.sort(resultFromOneScope, new ResultRecordComparator());
          treeModel = new VisibleTreeModel(resultRoot);
          datasetPanel.update(treeModel);        
      }
      
  }
}
