
/**
 *    '$RCSfile: SimpleChoiceLibrarySearchGUIPane.java,v $'
 *
 *     '$Author: berkley $'
 *       '$Date: 2004/10/13 21:55:10 $'
 *   '$Revision: 1.2 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2004 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

package org.ecoinformatics.seek.ecogrid;

/** 
 * This is hijacked from Chad Berkley's code
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;

import java.util.*;

import ptolemy.moml.ErrorHandler;
import ptolemy.moml.MoMLParser;
import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.Entity;
import ptolemy.kernel.util.IllegalActionException;
import ptolemy.kernel.util.NameDuplicationException;
import ptolemy.kernel.util.NamedObj;

/**
 * Class to build the search gui pane that allows the user to enter a single
 * search term to search by.
 */
public class SimpleChoiceLibrarySearchGUIPane 
    extends LibrarySearchPane implements ActionListener { 

    //constants
    private static final int EDGEGAP = 10;//GAP from container to component
    private static final int COMPONENTGAP = 6;//GAP from compent to component
    private static final int VERTICALEDGEGAP = 6;//GAP from container to

    // component
    private static final int VERTICALCOMPONENTGAP = 3;//GAP from component
                                                    // tocompent
    private static final String QUICKSEARCHLABEL = "Quick Search";
    private static final String GO = "Go";
    private static final String QUICKSEARCHACTION = "quickSearchAction";

    private BorderLayout topLayout = new BorderLayout();

    private JLabel quickSearchLabel = null;
    private JTextField searchValueField = null;
    private Vector searchScopeVector;
    private JButton quickSearchButton = null;
    private JCheckBox textCheckBox = null;
    private JCheckBox keywordCheckBox = null;
    private InputMap imap;
    private ActionMap amap;

    /**
     * constructor
     */
    public SimpleChoiceLibrarySearchGUIPane(ActionListener searchButtonHandler) {
	super(searchButtonHandler);
	init();
    }

    /**
     * @returns the search term
     */
    public String getSearchTerm() {
	return searchValueField.getText();
    }

    /**
     * @returns true if the text search type is selected
     */
    public boolean isTextSearchType() {
	return textCheckBox.isSelected();
    }

    /**
     * @returns true if the keyword search type is selected
     */
    public boolean isKeywordSearchType() {
	return keywordCheckBox.isSelected();
    }


    /*
     * Initialize the search panel
     */
    private void init() {
	BoxLayout searchPartLayout = new BoxLayout(this, BoxLayout.Y_AXIS);
	this.setLayout(searchPartLayout);

	// add a blank panel as border
	JPanel topBlankPanel = new JPanel();
	topBlankPanel.add(Box.createVerticalStrut(VERTICALEDGEGAP));
	this.add(topBlankPanel);

	// the panel contains label - Quick Search
	JPanel quickSearchLabelPanel = new JPanel();
	quickSearchLabelPanel.setLayout(new BoxLayout(quickSearchLabelPanel, BoxLayout.X_AXIS));
	quickSearchLabelPanel.add(Box.createHorizontalStrut(EDGEGAP));
	quickSearchLabel = new JLabel(QUICKSEARCHLABEL);
	quickSearchLabelPanel.add(quickSearchLabel);
	quickSearchLabelPanel.add(Box.createHorizontalGlue());
	// add the quickSearchLabelPanel to this
	this.add(quickSearchLabelPanel);

	// add a bank panel between quicksearch panel and
	// textFieldAndButton panel
	JPanel blankPanelUnderQuickSearch = new JPanel();
	blankPanelUnderQuickSearch.add(Box.createVerticalStrut(VERTICALCOMPONENTGAP));
	this.add(blankPanelUnderQuickSearch);

	// the panel contains the textfield and go button, it is has a
	// fixed height
	JPanel textFieldAndButtonPanel = new JPanel();
	textFieldAndButtonPanel.setLayout(new BoxLayout(textFieldAndButtonPanel, BoxLayout.X_AXIS));
	textFieldAndButtonPanel.add(Box.createHorizontalStrut(EDGEGAP));
	searchValueField = new JTextField();
	textFieldAndButtonPanel.add(searchValueField);
	textFieldAndButtonPanel.add(Box.createHorizontalStrut(COMPONENTGAP));
	// quickSearchAction
	quickSearchButton = new JButton(GO);
	quickSearchButton.addActionListener(this);
	textFieldAndButtonPanel.add(quickSearchButton);
	textFieldAndButtonPanel.add(Box.createHorizontalStrut(EDGEGAP));
	textFieldAndButtonPanel.add(Box.createHorizontalGlue());
	
	//add the textFieldAndButtonPanel into this
	this.add(textFieldAndButtonPanel);
	
	// create and add the search type check boxes
	textCheckBox = new JCheckBox("text");
	textCheckBox.setSelected(true);
	keywordCheckBox = new JCheckBox("keyword");
	keywordCheckBox.setSelected(true);
	// the panel
	JPanel quickSearchTypePanel = new JPanel();
	quickSearchTypePanel.setLayout(new BoxLayout(quickSearchTypePanel, BoxLayout.X_AXIS));
	quickSearchTypePanel.add(Box.createHorizontalStrut(EDGEGAP));
	quickSearchTypePanel.add(textCheckBox);
	quickSearchTypePanel.add(Box.createHorizontalStrut(COMPONENTGAP));
	quickSearchTypePanel.add(keywordCheckBox);
	quickSearchTypePanel.add(Box.createHorizontalStrut(EDGEGAP));
	quickSearchTypePanel.add(Box.createHorizontalGlue());
	this.add(quickSearchTypePanel);

	// add a blank panel below search type panel
	JPanel blankPanelUnderSearchType = new JPanel();
	blankPanelUnderSearchType.add(Box.createVerticalStrut(VERTICALCOMPONENTGAP));
	this.add(blankPanelUnderSearchType);
    }

    /**
     */
    public void actionPerformed(ActionEvent e) {
	// check if text box is selected ...
	if(!isTextSearchType() && !isKeywordSearchType()) {
	    // get the parent frame 
	    JFrame frame = getFrame();
	    // create a dialog-box warning
	    String errStr = "At least one search type (text or keyword) must be selected.";
	    String diaStr = "Quick-Search Error";
	    JOptionPane.showMessageDialog(frame, errStr, diaStr, JOptionPane.ERROR_MESSAGE);
	}
	else { 
	    // forward the event since one is selected
	    searchButtonHandler.actionPerformed(e);
	}
    }


    // helper function
    private JFrame getFrame() {
	Container c = getParent();
	while(!(c instanceof JFrame)) {
	    c = c.getParent();
	}
	return (JFrame)c;
    }



    /**
     * A factory that creates the searcher gui to search the library
     */
    public static class Factory extends LibrarySearchGUIPaneFactory {
	/** Create an factory with the given name and container.
	 *  @param container The container.
	 *  @param name The name of the entity.
	 *  @exception IllegalActionException If the container is incompatible
	 *   with this attribute.
	 *  @exception NameDuplicationException If the name coincides with
	 *   an attribute already in the container.
	 */
	public Factory(NamedObj container, String name)
            throws IllegalActionException, NameDuplicationException {
	    super(container, name);
	}

	/**
	 *  creates a ScrollBarModifier and returns it.
	 *  @param searchButtonHandler the ActionListener to handle the search
	 *  button events
	 *  @return A new LibrarySearchPane that displays the search pane
	 */
	public LibrarySearchPane createLibrarySearchGUIPane(ActionListener searchButtonHandler){
	    return new SimpleChoiceLibrarySearchGUIPane(searchButtonHandler);
	}

    }

} // SimpleChoiceLibrarySearchGUIPane
