/**
 *    '$RCSfile: SimpleLibrarySearchGUIPane.java,v $'
 *
 *     '$Author: berkley $'
 *       '$Date: 2004/09/08 19:09:21 $'
 *   '$Revision: 1.1 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 *    '$RCSfile: SimpleLibrarySearchGUIPane.java,v $'
 *
 *     '$Author: berkley $'
 *       '$Date: 2004/09/08 19:09:21 $'
 *   '$Revision: 1.1 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2004 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

package org.ecoinformatics.seek.ecogrid;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.BorderLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTree;
import javax.swing.ActionMap;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.InputMap;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.tree.TreeModel;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

import java.util.Stack;
import java.util.Vector;
import java.util.Iterator;

import ptolemy.moml.ErrorHandler;
import ptolemy.moml.MoMLParser;
import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.Entity;
import ptolemy.kernel.util.IllegalActionException;
import ptolemy.kernel.util.NameDuplicationException;
import ptolemy.kernel.util.NamedObj;

/**
 * Class to build the search gui pane that allows the user to enter a single
 * search term to search by.
 */
public class SimpleLibrarySearchGUIPane extends LibrarySearchPane
{

  //constants
  private static final int EDGEGAP = 10;//GAP from container to component
  private static final int COMPONENTGAP = 6;//GAP from compent to component
  private static final int VERTICALEDGEGAP = 6;//GAP from container to
                                               // component
  private static final int VERTICALCOMPONENTGAP = 3;//GAP from component
                                                    // tocompent
  private static final String QUICKSEARCHLABEL = "Quick Search";
  private static final String GO = "Go";
  private static final String QUICKSEARCHACTION = "quickSearchAction";

  private BorderLayout topLayout = new BorderLayout();

  private JLabel quickSearchLabel = null;
  private JTextField searchValueField = null;
  private Vector searchScopeVector;
  private JButton quickSearchButton = null;
  private InputMap imap;
  private ActionMap amap;

  /**
   * constructor
   */
  public SimpleLibrarySearchGUIPane(ActionListener searchButtonHandler)
  {
    super(searchButtonHandler);
    init();
  }

  /**
   * return the search term
   */
  public String getSearchTerm()
  {
    return searchValueField.getText();
  }

  /*
   * Initialize the search panel
   */
  private void init()
  {
    BoxLayout searchPartLayout = new BoxLayout(this,
                    BoxLayout.Y_AXIS);
    this.setLayout(searchPartLayout);
    //add a blank panel as border
    JPanel topBlankPanel = new JPanel();
    topBlankPanel.add(Box.createVerticalStrut(VERTICALEDGEGAP));
    this.add(topBlankPanel);

    // the panel contains label - Quick Search
    JPanel quickSearchLabelPanel = new JPanel();
    quickSearchLabelPanel.setLayout(new BoxLayout(quickSearchLabelPanel,
                    BoxLayout.X_AXIS));
    quickSearchLabelPanel.add(Box.createHorizontalStrut(EDGEGAP));
    quickSearchLabel = new JLabel(QUICKSEARCHLABEL);
    quickSearchLabelPanel.add(quickSearchLabel);
    quickSearchLabelPanel.add(Box.createHorizontalGlue());
    // add the quickSearchLabelPanel to this
    this.add(quickSearchLabelPanel);

    //add a bank panel between quicksearch panel and textFieldAndButton
    // panel
    JPanel blankPanelUnderQuickSearch = new JPanel();
    blankPanelUnderQuickSearch.add(Box
                    .createVerticalStrut(VERTICALCOMPONENTGAP));
    this.add(blankPanelUnderQuickSearch);

    // the panel contains the textfield and go button, it is has a fixed
    // height
    JPanel textFieldAndButtonPanel = new JPanel();
    textFieldAndButtonPanel.setLayout(new BoxLayout(
                    textFieldAndButtonPanel, BoxLayout.X_AXIS));
    textFieldAndButtonPanel.add(Box.createHorizontalStrut(EDGEGAP));
    searchValueField = new JTextField();
    textFieldAndButtonPanel.add(searchValueField);
    textFieldAndButtonPanel.add(Box.createHorizontalStrut(COMPONENTGAP));
    // quickSearchAction

    quickSearchButton = new JButton(GO);
    quickSearchButton.addActionListener(searchButtonHandler);
    textFieldAndButtonPanel.add(quickSearchButton);
    textFieldAndButtonPanel.add(Box.createHorizontalStrut(EDGEGAP));
    textFieldAndButtonPanel.add(Box.createHorizontalGlue());

    //add the textFieldAndButtonPanel into this
    this.add(textFieldAndButtonPanel);

    //add a bank panel between quicksearch panel and textFieldAndButton
    // panel
    JPanel blankPanelUnderTextField = new JPanel();
    blankPanelUnderTextField.add(Box
                    .createVerticalStrut(VERTICALCOMPONENTGAP));
    this.add(blankPanelUnderTextField);

    //add a bank panel between quicksearch panel and textFieldAndButton
    // panel
    JPanel blankPanelUnderAdvanceSearch = new JPanel();
    blankPanelUnderAdvanceSearch.add(Box
                    .createVerticalStrut(VERTICALEDGEGAP));
    this.add(blankPanelUnderAdvanceSearch);
  }

  /**
   * A factory that creates the searcher gui to search the library
   */
  public static class Factory extends LibrarySearchGUIPaneFactory
  {
    /** Create an factory with the given name and container.
     *  @param container The container.
     *  @param name The name of the entity.
     *  @exception IllegalActionException If the container is incompatible
     *   with this attribute.
     *  @exception NameDuplicationException If the name coincides with
     *   an attribute already in the container.
     */
    public Factory(NamedObj container, String name)
            throws IllegalActionException, NameDuplicationException
    {
      super(container, name);
    }

    /**
     *  creates a ScrollBarModifier and returns it.
     *  @param searchButtonHandler the ActionListener to handle the search
     *  button events
     *  @return A new LibrarySearchPane that displays the search pane
     */
    public LibrarySearchPane createLibrarySearchGUIPane(
      ActionListener searchButtonHandler)
    {
      return new SimpleLibrarySearchGUIPane(searchButtonHandler);
    }
  }
}
