/**
 *    '$RCSfile: QBApp.java,v $'
 *
 *     '$Author: rspears $'
 *       '$Date: 2004/09/20 15:05:50 $'
 *   '$Revision: 1.2 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2003 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */
package org.ecoinformatics.seek.querybuilder;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GraphicsConfiguration;
import java.awt.GraphicsEnvironment;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.Vector;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JEditorPane;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.SingleSelectionModel;
import javax.swing.SwingUtilities;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;

import org.ecoinformatics.seek.datasource.Attribute;
import org.ecoinformatics.seek.datasource.DSSchemaDef;
import org.ecoinformatics.seek.datasource.DSSchemaIFace;
import org.ecoinformatics.seek.datasource.DSTableDef;
import org.ecoinformatics.seek.datasource.DataType;
import org.ecoinformatics.seek.datasource.TableEntity;

/**
 * The Query Builder app which extends a JPanel. This enables it to be embedded in an applet or an application.
 */
public class QBApp extends JPanel implements ChangeListener, TableModelListener
{

  // The preferred size of the demo
  private static final int    PREFERRED_WIDTH  = 720;
  private static final int    PREFERRED_HEIGHT = 640;
  private static final String ADD_MSG          = "Add an item by clicking in the empty \"table\" cell, or drag a \"field\" item from the top panel to the bottom.";
  private static final String CHANGED_MSG      = "Query has Changed.";

  private boolean             mDoingDemo      = false;
  private boolean             mDoingDemoTests = false;

  private JPanel              mDemoPanel   = null;

  private boolean             mIsChanged   = false;
  private boolean             mNoOuterUI   = false;
   

  // Status Bar
  private JTextField          mStatusField = null;
  private JMenuBar            mMenuBar     = null;
  private JFrame              mFrame       = null;
  private QBApplet            mApplet      = null;

  private JTabbedPane         mTabbedPane  = null;
  private int                 mTabInx      = 0;
  private int                 mLastTabInx  = 0;

  private DSSchemaIFace       mSchema      = null;
  private DBQueryDef          mQueryDef    = null;

  private QBSplitPaneStandard mStdSP       = null;
  private QBSplitPaneAdvanced mAdvSP       = null;
  private QBSplitPaneSQL      mSQLSP       = null;
  private TableModelListener  mExternalTMListener = null;

  /**
   * Constructor with Applet
   * @param aApplet
   */
  public QBApp()
  {
    mNoOuterUI = true;
    initialize();
  }

  /**
   * Constructor with Applet
   * @param aApplet
   */
  public QBApp(QBApplet aApplet)
  {
    this(aApplet, null);
  }

  /**
   * QBApp Constructor
   */
  public QBApp(QBApplet aApplet, GraphicsConfiguration gc)
  {

    // Note that the applet may null if this is started as an application
    mApplet = aApplet;

    // Create Frame here for app-mode
    if (!isApplet() && gc != null)
    {
      mFrame = new JFrame(gc);
    }
    initialize();
  }
  
  /**
   * General Method for initializing the class
   *
   */
  private void initialize()
  {
    setLayout(new BorderLayout());

    // set the preferred size of the demo
    setPreferredSize(new Dimension(PREFERRED_WIDTH, PREFERRED_HEIGHT));


    initializeUI();

    JEditorPane editorPane = new JEditorPane("text/text", "Generated SQL will go here!");
    editorPane.setEditable(false);

    // create tab
    mTabbedPane  = new JTabbedPane();

    DBQueryDef queryDef = null;
    if (mDoingDemo)
    {
      queryDef = createDemoQuery();
      loadSchemaIntoUI(createDemoSchema());
    }

    add(mTabbedPane, BorderLayout.CENTER);

    setStatus(ADD_MSG);

    if (mDoingDemo)
    {
      loadQuery(queryDef);
      //mIsChanged = false;
      //mLastTabInx = mQueryDef.isAdv() ? 1 : 0;
      //mTabbedPane.setSelectedIndex(mLastTabInx);
    }

    // Note that
    // we again must do this on the GUI thread using invokeLater.
    SwingUtilities.invokeLater(new Runnable()
    {
      public void run()
      {
        showQBApp();
       }
    });
  }

  /**
   * Sets a new Schema into the UI and recreates the UI for it
   * @param aSchema the new schema
   */
  public void loadSchemaIntoUI(DSSchemaIFace aSchema)
  {
    if (aSchema == null) return;

    mSchema = aSchema;

    if (mSQLSP != null)
    {
      mTabbedPane.getModel().removeChangeListener(mSQLSP);
    }

    //boolean firstTime = mStdSP == null;

    if (mAdvSP != null)
      mAdvSP.shutdown();

    mStdSP = null;
    mAdvSP = null;
    mSQLSP = null;

    mTabbedPane.removeAll();

/*    if (!isApplet())
    {
      java.lang.System.gc();
      java.lang.System.runFinalization();
      java.lang.System.runFinalizersOnExit(true);
    }*/

    //if (!firstTime) return;

    mStdSP = new QBSplitPaneStandard(mSchema);
    mAdvSP = new QBSplitPaneAdvanced(mSchema, this);
    mSQLSP = new QBSplitPaneSQL(mStdSP, mAdvSP);

    // Initialize
    mStdSP.setTableModelListener(this);
    mAdvSP.setTableModelListener(this);

    mTabbedPane.add("Standard", mStdSP);
    mTabbedPane.add("Advanced", mAdvSP);
    mTabbedPane.add("SQL", mSQLSP);

    mTabbedPane.getModel().addChangeListener(mSQLSP);
    mTabbedPane.getModel().addChangeListener(this);
  }

  /**
   *
   * @return returns a test/demo schema
   */
  private DSSchemaDef createDemoSchema()
  {
    DSSchemaDef schema = null;
    if (mDoingDemo) {
      schema = new DSSchemaDef("Test Schema");
      boolean useTableEntity = true;
      if (useTableEntity)
      {
        TableEntity table1 = new TableEntity("", "Employees", "", new Boolean(true), TableEntity.ROWMAJOR, 0);
        table1.add(new Attribute("", "EmpNo", DataType.INT));
        table1.add(new Attribute("", "First Name",  DataType.STR));
        table1.add(new Attribute("", "Last Name",  DataType.STR));
        table1.add(new Attribute("", "Addr 1",  DataType.STR));
        table1.add(new Attribute("", "Addr 2",  DataType.STR));
        table1.add(new Attribute("", "City",  DataType.STR));
        table1.add(new Attribute("", "State",  DataType.STR));
        table1.add(new Attribute("", "Zip",  DataType.STR));
        schema.addTable(table1);

        table1 = new TableEntity("", "EmpDept", "", new Boolean(true), TableEntity.ROWMAJOR, 0);
        table1.add(new Attribute("", "EmpNo", DataType.INT));
        table1.add(new Attribute("", "DeptNo", DataType.INT));
        schema.addTable(table1);

        table1 = new TableEntity("", "Department", "", new Boolean(true), TableEntity.ROWMAJOR, 0);
        table1.add(new Attribute("", "DeptNo", DataType.INT));
        table1.add(new Attribute("", "Name",  DataType.STR));
        for (int i=0;i<15;i++) {
          table1.add(new Attribute("", "User Field "+i,  DataType.STR));
        }
        schema.addTable(table1);

        table1 = new TableEntity("", "EmpDoc", "", new Boolean(true), TableEntity.ROWMAJOR, 0);
        table1.add(new Attribute("", "EmpNo", DataType.INT));
        table1.add(new Attribute("", "DocNo", DataType.INT));
        schema.addTable(table1);

        table1 = new TableEntity("", "Document", "", new Boolean(true), TableEntity.ROWMAJOR, 0);
        table1.add(new Attribute("", "DocNo", DataType.INT));
        table1.add(new Attribute("", "Title", DataType.STR));
        table1.add(new Attribute("", "Desc", DataType.STR));
        table1.add(new Attribute("", "Biblo", DataType.STR));
        schema.addTable(table1);

      }
      else
      {
        DSTableDef table1 = new DSTableDef("Employees");
        table1.addPrimaryKey("EmpNo", DataType.INT);
        table1.addField("First Name",  DataType.STR);
        table1.addField("Last Name",  DataType.STR);
        table1.addField("Addr 1",  DataType.STR);
        table1.addField("Addr 2",  DataType.STR);
        table1.addField("City",  DataType.STR);
        table1.addField("State",  DataType.STR);
        table1.addField("Zip",  DataType.STR);
        schema.addTable(table1);

        table1 = new DSTableDef("EmpDept");
        table1.addPrimaryKey("EmpNo", DataType.INT);
        table1.addSecondaryKey("DeptNo", DataType.INT);
        schema.addTable(table1);

        table1 = new DSTableDef("Department");
        table1.addPrimaryKey("DeptNo", DataType.INT);
        table1.addField("Name",  DataType.STR);
        for (int i=0;i<15;i++) {
          table1.addField("User Field "+i,  DataType.STR);
        }
        schema.addTable(table1);

        table1 = new DSTableDef("EmpDoc");
        table1.addPrimaryKey("EmpNo", DataType.INT);
        table1.addSecondaryKey("DocNo", DataType.INT);
        schema.addTable(table1);

        table1 = new DSTableDef("Document");
        table1.addPrimaryKey("DocNo", DataType.INT);
        table1.addField("Title", DataType.STR);
        table1.addField("Desc", DataType.STR);
        table1.addField("Biblo", DataType.STR);
        schema.addTable(table1);
      }
    }
    return schema;
  }

  /**
   * 
   * @return returns a demo query object
   */
  private DBQueryDef createDemoQuery()
  {
    DBQueryDef queryDef = new DBQueryDef();
    Vector joins = new Vector();
    String tableNames[] = {"Employees", "EmpDept", "EmpDept", "Department",
                           "Employees", "EmpDoc", "EmpDoc", "Document"};
    String fieldNames[] = {"EmpNo", "EmpNo","DeptNo", "DeptNo",
                           "EmpNo", "EmpNo","DocNo", "DocNo"};
    int    tableIds[]   = {0,1, 1,2, 0,3, 3,4};
    for (int i=0;i<tableNames.length;i++)
    {
      DBSelectTableModelItem item = new DBSelectTableModelItem(tableNames[i], fieldNames[i], "", false, "", "");
      item.setTableId(tableIds[i]);
      joins.add(item);
    }
    queryDef.setJoins(joins);

    int    tableFrameIds[]   = {0,1,2,3,4};
    String tableFrameNames[] = {"Employees", "EmpDept", "Department", "EmpDoc", "Document"};
    for (int i=0;i<tableFrameIds.length;i++)
    {
      queryDef.addTable(tableFrameIds[i], tableFrameNames[i], -1, -1);
    }
    queryDef.setIsAdv(true);
    return queryDef;
  }

  // *******************************************************
  // *************** Load UI ******************
  // *******************************************************

  public void initializeUI()
  {

    JPanel top = new JPanel();
    top.setLayout(new BorderLayout());
    add(top, BorderLayout.NORTH);

    mMenuBar = createMenus();
    if (mMenuBar != null)
      top.add(mMenuBar, BorderLayout.NORTH);

    mStatusField = new JTextField("");
    mStatusField.setEditable(false);
    add(mStatusField, BorderLayout.SOUTH);

    mDemoPanel = new JPanel();
    mDemoPanel.setLayout(new BorderLayout());
    mDemoPanel.setBorder(new EtchedBorder());
    mDemoPanel.setBackground(Color.green);
    add(mDemoPanel, BorderLayout.CENTER);

  }

  /**
   * Create menus
   */
  public JMenuBar createMenus()
  {
    JMenuBar menuBar = null;

    if (!isApplet() && !mNoOuterUI)
    {
      menuBar = new JMenuBar();
      JMenu fileMenu = (JMenu) menuBar.add(new JMenu("File"));
      fileMenu.setMnemonic('F');
      createMenuItem(fileMenu, "Open Query", "O", "Open Query File", new GenericMenuAction(this, GenericMenuAction.OPEN_QUERY_ACTION));
      createMenuItem(fileMenu, "Save Query", "S", "Save Query", new GenericMenuAction(this, GenericMenuAction.SAVE_QUERY_ACTION));
      fileMenu.addSeparator();
      createMenuItem(fileMenu, "Open Schema", "O", "Open Schema File", new GenericMenuAction(this, GenericMenuAction.OPEN_SCHEMA_ACTION));
      createMenuItem(fileMenu, "Save Schema", "S", "Save Schema", new GenericMenuAction(this, GenericMenuAction.SAVE_SCHEMA_ACTION));
      fileMenu.addSeparator();
      createMenuItem(fileMenu, "Exit", "x", "Exit Appication", new GenericMenuAction(this, GenericMenuAction.EXIT_ACTION));

      if (mDoingDemoTests)
      {
        fileMenu = (JMenu) menuBar.add(new JMenu("Tests"));
        fileMenu.setMnemonic('T');
        createMenuItem(fileMenu, "Test Queries", "e", "Test Queries", new GenericMenuAction(this, GenericMenuAction.QUERY_TESTS_ACTION));
      }
    }

    return menuBar;
  }

  /**
   * Creates a generic menu item
   */
  public JMenuItem createMenuItem(JMenu menu,
                                  String label,
                                  String mnemonic,
                                  String accessibleDescription,
                                  Action action)
  {
    JMenuItem mi = (JMenuItem) menu.add(new JMenuItem(label));
    mi.setMnemonic(mnemonic.charAt(0));
    mi.getAccessibleContext().setAccessibleDescription(accessibleDescription);
    mi.addActionListener(action);
    if (action == null)
    {
      mi.setEnabled(false);
    }
    return mi;
  }


  /**
   * Bring up the QBApp demo by showing the frame (only
   * applicable if coming up as an application, not an applet);
   */
  public void showQBApp()
  {
    if (!isApplet() && getFrame() != null)
    {
      // put QBApp in a frame and show it
      JFrame f = getFrame();
      f.setTitle("Query Builder");
      f.getContentPane().add(this, BorderLayout.CENTER);
      f.pack();

      Rectangle screenRect = f.getGraphicsConfiguration().getBounds();
      Insets screenInsets = Toolkit.getDefaultToolkit().getScreenInsets(
          f.getGraphicsConfiguration());

      // Make sure we don't place the demo off the screen.
      int centerWidth = screenRect.width < f.getSize().width ? screenRect.x : screenRect.x
          + screenRect.width / 2 - f.getSize().width / 2;
      int centerHeight = screenRect.height < f.getSize().height ? screenRect.y : screenRect.y
          + screenRect.height / 2 - f.getSize().height / 2;

      centerHeight = centerHeight < screenInsets.top ? screenInsets.top : centerHeight;

      f.setLocation(centerWidth, centerHeight);
      f.show();
    }
  }

  /**
   * Determines if this is an applet or application
   */
  public boolean isApplet()
  {
    return (mApplet != null);
  }

  /**
   * Returns the applet instance
   */
  public QBApplet getApplet()
  {
    return mApplet;
  }

  /**
   * Returns the frame instance
   */
  public JFrame getFrame()
  {
    return mFrame;
  }

  /**
   * Returns the menubar
   */
  public JMenuBar getMenuBar()
  {
    return mMenuBar;
  }

  /**
   * Set the status
   */
  public void setStatus(String s)
  {
    // do the following on the gui thread
    SwingUtilities.invokeLater(new QBAppRunnable(this, s)
    {
      public void run()
      {
        mQBApp.mStatusField.setText((String) obj);
      }
    });
  }
  
  public void setExternalTMListener(TableModelListener aL)
  {
    mExternalTMListener = aL;
  }

  /**
   * Saves Query to a file
   */
  public Writer saveFile(String aTitle, String aFilterStr, String aFilterTitle)
  {
    JFileChooser chooser = new JFileChooser("Save "+aTitle);
    chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
    chooser.setSelectedFile(null);
    chooser.addChoosableFileFilter(new DBFileFilter(aFilterStr, aFilterTitle));
    int retval = chooser.showSaveDialog(this);
    if (retval == JFileChooser.APPROVE_OPTION)
    {
      //declared here only to make visible to finally clause; generic reference
      Writer output = null;
      try
      {
        //use buffering
        return new BufferedWriter(new FileWriter(chooser.getSelectedFile()));
      }
            catch (FileNotFoundException ex) {
              JOptionPane.showMessageDialog(this, ex.toString());
            }
            catch (IOException ex){
              JOptionPane.showMessageDialog(this, ex.toString());
            }
            finally
      {
        //flush and close both "output" and its underlying FileWriter
              try
              {
                if (output != null)
                  output.close();
              }
                    catch (IOException ex) {
                JOptionPane.showMessageDialog(this, ex.toString());
                    }
      }
    }
    return null;
  }

  /**
   * Saves Query to a file
   */
  public void saveQueryFile()
  {
    Writer output = saveFile("Query", "query", "Query Files");
    if (output != null)
    {
            DBQueryDef queryDef = new DBQueryDef();
            if (mTabbedPane.getSelectedIndex() == 0)
            {
              mStdSP.fillQueryDef(queryDef);
            }
            else
            {
              mAdvSP.fillQueryDef(queryDef);
            }

      try
      {
        output.write(DBQueryDefParserEmitter.emitXML(queryDef));
        output.close();
        mIsChanged = false;
        setStatus(ADD_MSG);
      }
            catch (IOException ex)
            {
              JOptionPane.showMessageDialog(this, ex.toString());
            }
    }
  }

  /**
   * Saves Query to a file
   */
  public void saveSchemaFile()
  {
    Writer output = saveFile("Schema", "schema", "QueSchemary Files");
    if (output != null)
    {
      try
      {
        output.write(DBSchemaParserEmitter.emitXML(mSchema));
        output.close();
      }
            catch (IOException ex)
            {
              JOptionPane.showMessageDialog(this, ex.toString());
            }
    }
  }

  /**
   * Loads the QueryDef object into the UI
   * @param aQueryDef the query def
   */
  protected void loadQuery(DBQueryDef aQueryDef)
  {
    if (aQueryDef != null)
    {
      mQueryDef = aQueryDef;

      int status = mStdSP.buildFromQueryDef(aQueryDef);
      if (status == DBQueryDef.BUILD_TOO_COMPLEX_WHERE ||
          status == DBQueryDef.BUILD_TOO_COMPLEX_JOINS)
      {
        mTabbedPane.setSelectedIndex(1);
        JOptionPane.showMessageDialog(this, "Switching to the Advanced View,\nthe query is too complex for the Standard Tab.");
      } else 
      {
          mLastTabInx = mQueryDef.isAdv() ? 1 : 0;
          mTabbedPane.setSelectedIndex(mLastTabInx);
      }
      mAdvSP.buildFromQueryDef(aQueryDef);
    }
    else
    {
      JOptionPane.showMessageDialog(this, "There was an error parsing the query.\nIt could not be loaded.");
    }
    mIsChanged = false;
    setStatus(ADD_MSG);
  }
  
  /**
   * Fills QueryDef from Model
   */
  public void fillQueryDef(DBQueryDef aQueryDef)
  {
    if (aQueryDef == null) return;

    // Tab Inx '2' is the SQL Pane so we need to use the 'last" one we were at
    int tabInx = mTabInx == 2 ? mLastTabInx : mTabInx;
    
    if (tabInx == 0)
      mStdSP.fillQueryDef(aQueryDef);
    else
      mAdvSP.fillQueryDef(aQueryDef);
  }
  

  /**
   * Loads a Schema and a Query into the Builder
   * @param aSchema The schema to be used
   * @param aQueryDef the query def to be loaded (can be null)
   */
  public void set(DSSchemaIFace aSchema, DBQueryDef aQueryDef)
  {
    mQueryDef = aQueryDef;
    loadSchemaIntoUI(aSchema);
    if (mQueryDef != null)
    {
      loadQuery(mQueryDef);
    }
  }

  /**
   *
   * Opens a Query and loads into the builder
   */
  public String openFile(String aTitle, String aFilterStr, String aFilterTitle)
  {
    JFileChooser chooser = new JFileChooser("Open "+aTitle);
    chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
    chooser.setSelectedFile(null);
    chooser.addChoosableFileFilter(new DBFileFilter(aFilterStr, aFilterTitle));
          JComponent frame = this;

        // clear the preview from the previous display of the chooser
    int retval = chooser.showDialog(frame, null);
    if (retval == JFileChooser.APPROVE_OPTION)
    {
      File theFile = chooser.getSelectedFile();
      if (theFile != null)
      {
        if (theFile.isDirectory())
        {
          //JOptionPane.showMessageDialog(frame, "You chose this directory: " + theFile.getPath());
        }
        else
        {
          return theFile.getPath();
        }
      }

    } else if (retval == JFileChooser.CANCEL_OPTION)
    {
      //JOptionPane.showMessageDialog(frame, "User cancelled operation. No file was chosen.");
    } else if (retval == JFileChooser.ERROR_OPTION)
    {
      JOptionPane.showMessageDialog(frame, "An error occured. No file was chosen.");
    } else
    {
      JOptionPane.showMessageDialog(frame, "Unknown operation occured.");
    }
    return null;
  }

  /**
   *
   * Opens a Query and loads into the builder
   */
  public void openQueryFile()
  {
    String path = openFile("Query", "query", "Query Files");
    if (path != null)
    {
      mQueryDef = DBQueryDefParserEmitter.readQueryDef(mSchema, path);
      if (mQueryDef == null)
      {
        JOptionPane.showMessageDialog(this, DBQueryDefParserEmitter.getErrorCodeText(), "Query Error", JOptionPane.ERROR_MESSAGE);
      } else
      {
        loadQuery(mQueryDef);
      }
    }
   }

  /**
   *
   * Opens a Query and loads into the builder
   */
  public void openSchemaFile()
  {
    String path = openFile("Schema", "schema", "Schema Files");
    if (path != null)
    {

      DSSchemaIFace schema = DBSchemaParserEmitter.readSchemaDef(path);
      if (schema != null)
      {
        mQueryDef = null;
        loadSchemaIntoUI(schema);
      }
    }
  }

  /**
   * Helper method
   * @param aFileName
   */
  private void runQueryTest(String aFileName)
  {
    DBQueryDef queryDef = DBQueryDefParserEmitter.readQueryDef(mSchema, aFileName);
    System.err.print(aFileName + " - ");
    System.err.println(queryDef == null ? DBQueryDefParserEmitter.getErrorCodeText() : "OK");

  }

  /**
   *
   * Loads various queries for testing the parser
   */
  public void testQueries()
  {
    String dirStr = "/home/globus/";

    runQueryTest(dirStr + "good.query");
    runQueryTest(dirStr + "join_fieldname_bad.query");
    //runQueryTest(dirStr + "join_tableid_bad.query");
    runQueryTest(dirStr + "join_tablename_bad.query");
    runQueryTest(dirStr + "select_fieldname_bad.query");
    runQueryTest(dirStr + "select_tablename_bad.query");
    runQueryTest(dirStr + "table_tablename_bad.query");
    runQueryTest(dirStr + "where_fieldname_bad.query");
    runQueryTest(dirStr + "where_tablename_bad.query");
  }

  // *******************************************************
  // ******************   Runnables  ***********************
  // *******************************************************

  /**
   * Generic QBApp runnable. This is intended to run on the
   * AWT gui event thread so as not to muck things up by doing
   * gui work off the gui thread. Accepts a QBApp and an Object
   * as arguments, which gives subtypes of this class the two
   * "must haves" needed in most runnables for this demo.
   */
  class QBAppRunnable implements Runnable
  {

    protected QBApp mQBApp;

    protected Object    obj;

    public QBAppRunnable(QBApp aQBApp, Object obj)
    {
      this.mQBApp = aQBApp;
      this.obj = obj;
    }

    public void run()
    {
    }
  }

  // *******************************************************
  // ********************   Actions  ***********************
  // *******************************************************


  class GenericMenuAction extends AbstractAction
  {
    public static final int OPEN_QUERY_ACTION  = 0;
    public static final int SAVE_QUERY_ACTION  = 1;
    public static final int OPEN_SCHEMA_ACTION = 2;
    public static final int SAVE_SCHEMA_ACTION = 3;
    public static final int EXIT_ACTION        = 4;
    public static final int QUERY_TESTS_ACTION = 5;


    QBApp mQueryBuilder;
    int   mActionType;

    protected GenericMenuAction(QBApp aQBApp, int aActionType)
    {
      super("Action");
      mQueryBuilder = aQBApp;
      mActionType = aActionType;
    }

    public void actionPerformed(ActionEvent e)
    {
      switch (mActionType)
      {
        case OPEN_QUERY_ACTION:
          mQueryBuilder.openQueryFile();
          break;
        case SAVE_QUERY_ACTION:
          mQueryBuilder.saveQueryFile();
          break;
        case OPEN_SCHEMA_ACTION:
          mQueryBuilder.openSchemaFile();
          break;
        case SAVE_SCHEMA_ACTION:
          mQueryBuilder.saveSchemaFile();
          break;
        case EXIT_ACTION:
          {
            if (mIsChanged)
            {
              int x0 = JOptionPane.CLOSED_OPTION;
              int x1 = JOptionPane.YES_OPTION;
              int x2 = JOptionPane.NO_OPTION;
              int x3 = JOptionPane.OK_OPTION;
              int x4 = JOptionPane.CLOSED_OPTION;
              Object[] options = { "Discard and Exit", "Save and Exit", "Cancel" };
              String msg = "You have changed the query, do you wish to save it before exiting?";
              int option = JOptionPane.showOptionDialog(null, msg, "Exiting",
                  JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
                  null, options, options[1]);
              if (option == JOptionPane.NO_OPTION)
              {
                mQueryBuilder.saveQueryFile();
              } else if (option == 2)
              {
                return;
              }
            }
            System.exit(0);
          }
          break;
        case QUERY_TESTS_ACTION:
          mQueryBuilder.testQueries();
          break;
      }

    }
  }

  //-----------------------------------------------------------------------------
  //--  ChangeListener Interface
  //-----------------------------------------------------------------------------

  /**
   * Notify when TAB changes
  */
  public void stateChanged(ChangeEvent e)
  {
    if (mQueryDef == null) return;

    Object[] options = { "Standard (data loss)", "Advanced" };
    int oldTabInx = mTabInx;
    SingleSelectionModel model = (SingleSelectionModel) e.getSource();
    mTabInx = model.getSelectedIndex();

    if (mQueryDef == null)
    {
      mLastTabInx = mTabInx;
      return;
    }

    // mIsChanged gets sets during the "loading" of a query
    // so we cache it here and then set it back
    boolean cachedIsChanged = mIsChanged;
    String msgStr = "";
    switch (mTabInx) {
      case 0 :
        {
          if (mAdvSP.possibleDataLoss())
          {
            String msg = "The \"where\" clause doesn't have any \"select\" items\ndata loss will occrred when switching to the Standard View.\nPlease choose a view:";
            int option = JOptionPane.showOptionDialog(null, msg, "No Display Items",
                JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE, null, options, options[0]);
            if (option == JOptionPane.NO_OPTION)
            {
              mTabbedPane.setSelectedIndex(1);
              return;
            }
          }
          if (mLastTabInx != 0)
          {
            if (mIsChanged)
            {
              mQueryDef = new DBQueryDef();
              mAdvSP.fillQueryDef(mQueryDef);
            }
            System.out.println(DBQueryDefParserEmitter.emitXML(mQueryDef)); // XXX
                                                                            // debug
                                                                            // only
            int status = mStdSP.buildFromQueryDef(mQueryDef);
            if (status != DBQueryDef.BUILD_OK)
            {
              if (status == DBQueryDef.BUILD_ERROR)
              {
                JOptionPane.showMessageDialog(this,
                    "There was an error and the query could be build.", "Error",
                    JOptionPane.ERROR_MESSAGE);
              } else
              {
                String msg = "Undefined Error";
                switch (status) {
                  case DBQueryDef.BUILD_TOO_COMPLEX_WHERE :
                    msg = "The \"where\" clause is too complicated for the Standard View.\nPlease choose a view:";
                  case DBQueryDef.BUILD_TOO_COMPLEX_JOINS :
                    msg = "The \"join\" and \"where\" are too complicated for the Standard View.\nPlease choose a view:";
                    break;
                }
                int option = JOptionPane.showOptionDialog(null, msg, "Problem",
                    JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE, null, options,
                    options[0]);
                if (option == JOptionPane.NO_OPTION)
                {
                  mTabbedPane.setSelectedIndex(1);
                }
              }
            }
          }
          msgStr = ADD_MSG;
          mLastTabInx = 0;
        }
        break;
      case 1 :
        {
          if (mLastTabInx != 1)
          {
            if (mIsChanged)
            {
              mQueryDef = new DBQueryDef();
              mStdSP.fillQueryDef(mQueryDef);
            }
            System.out.println(DBQueryDefParserEmitter.emitXML(mQueryDef));
            mAdvSP.buildFromQueryDef(mQueryDef);
          }
          msgStr = ADD_MSG;
          mLastTabInx = 1;
        }
        break;
      case 2 :
        // no-op
        break;
    }

    mIsChanged = cachedIsChanged;
    setStatus(mIsChanged? CHANGED_MSG : msgStr);

  }

  //-----------------------------------------------------------------------------
  //-- TableModelListener Interface
  //-----------------------------------------------------------------------------

  public void tableChanged(TableModelEvent e)
  {
    mIsChanged = true;
    setStatus(CHANGED_MSG);
    if (mExternalTMListener != null) 
    {
      mExternalTMListener.tableChanged(e);
    }
  }

  //-----------------------------------------------------------------------------
  //-- Application MAIN
  //-----------------------------------------------------------------------------

  /**
   * QBApp Main. Called only if we're an application, not an applet.
   */
  public static void main(String[] args)
  {
    // Create QBApp on the default monitor
    QBApp qbApp = new QBApp(null, GraphicsEnvironment.getLocalGraphicsEnvironment()
        .getDefaultScreenDevice().getDefaultConfiguration());
  }

}

