/**
 *    '$RCSfile: DebugPrinter.java,v $'
 *
 *     '$Author: rspears $'
 *       '$Date: 2004/09/20 15:05:50 $'
 *   '$Revision: 1.6 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2003 The Regents of the University of California.
 * All rights reserved.
 * 
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

package org.ecoinformatics.util;

/**
 * a class that prints debug statement based on a system property passed in
 * at runtime.  The property name DBGPRN can be set to 3 values (0, 1, 2).
 * When set to 0, no debug lines are printed.  When set to 1, only the most
 * important lines are printed, and when set to 2, everthing is printed.
 */
public class DebugPrinter
{
  private static DebugPrinter printer = null;
  private int DEBUG_LEVEL = 2;

  /**
   * constructor.  this should not be used.  use getInstance() instead.
   */
  protected DebugPrinter()
  {
    String dbg = System.getProperty("DBGPRN");
    try
    {
      if(dbg != null)
      {
        Integer I = new Integer(dbg);
        DEBUG_LEVEL = I.intValue();
      }
      else
      {
        System.out.println("DEBUG_LEVEL is defaulting to 0 (no output) " +
        "because the " +
        "environment variable DBGPRN was not set.");
      }
    }
    catch(ClassCastException cce)
    {
      System.out.println("DEBUG_LEVEL is defaulting to 0 (no output) " +
        "because the " +
        "environment variable DBGPRN was either not set or was not set " +
        "to an integer.");
    }

    if(DEBUG_LEVEL > 2 || DEBUG_LEVEL < 0)
    {
      System.out.println("DEBUG_LEVEL is defaulting to 0 (no output) " +
        "because the " +
        "environment variable DBGPRN was not set to 0, 1 or 2.");
      DEBUG_LEVEL=0;
    }
  }

  /**
   * creates a singleton instance of this class
   */
  public static DebugPrinter getInstance()
  {
    if(printer == null)
    {
      printer = new DebugPrinter();
    }
    return printer;
  }

  /**
   * prints a message based on its level.  if level <= DBGPRN then it prints
   * @param msg the message to print
   * @param level the level to apply to the message (1 or 2).  Set your level
   * to 1 if the message is of a higher importance.  Set the level to 2 if
   * it is a general debug statement
   */
  public void print(String msg, int level)
  {
    if(level == 0)
    {
      return;
    }
    else if(level <= DEBUG_LEVEL)
    {
      System.out.println(msg);
    }
  }
}
