/* Web service harvester....
 Copyright (c) 2000-2003 The Regents of the University of California.
 All rights reserved.
 Permission is hereby granted, without written agreement and without
 license or royalty fees, to use, copy, modify, and distribute this
 software and its documentation for any purpose, provided that the above
 copyright notice and the following two paragraphs appear in all copies
 of this software.
 IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
 THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.
 THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
 CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 ENHANCEMENTS, OR MODIFICATIONS.
                                        PT_COPYRIGHT_VERSION_2
                                        COPYRIGHTENDKEY
 */

package org.sdm.spa;

import ptolemy.actor.TypedAtomicActor;
import ptolemy.actor.TypedIOPort;
import ptolemy.data.StringToken;
import ptolemy.data.type.BaseType;
import ptolemy.data.type.Type;
import ptolemy.kernel.attributes.FileAttribute;
import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.util.*;
import ptolemy.gui.MessageHandler;

// For IO and URL connections
import java.io.Reader;
import java.io.Writer;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.InputStreamReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.Iterator;
import java.io.StringReader;
import java.net.HttpURLConnection;
import java.net.URL;

// For JAXP
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.rpc.ParameterMode;
import javax.xml.rpc.ServiceException;
import javax.xml.namespace.QName;
import org.xml.sax.InputSource;

// For Xalan XPath and AXIS API
import org.apache.xpath.XPathAPI;
import org.w3c.dom.traversal.NodeIterator;
import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.apache.axis.AxisFault;
import org.apache.axis.client.Call;
import org.apache.axis.client.Service;
import org.apache.axis.encoding.XMLType;
import org.apache.axis.utils.Options;

//////////////////////////////////////////////////////////////////////////
//// WSHarvester
/** Given a repository as a parameter, this actor generates a folders for all
    the web services in this repository under the superfolder called
    "webservices". Under each web service folder, it generates actors for all
    the operations of the webservice specified by its wsdl.
    @author Ilkay Altintas
    @version $Id: WSHarvester.java,v 1.15 2004/10/14 00:29:12 altintas Exp $
 */

public class WSHarvester
    extends TypedAtomicActor {
  /** Construct a WebService actor with the given container and name.
   *  @param container The container.
   *  @param name The name of this actor.
   *  @exception IllegalActionException If the actor cannot be contained
   *   by the proposed container.
   *  @exception NameDuplicationException If the container already has an
   *   actor with this name.
   */
  public WSHarvester(CompositeEntity container, String name) throws
      NameDuplicationException, IllegalActionException {

    super(container, name);

    result = new TypedIOPort(this, "result", false, true);
    result.setTypeEquals(BaseType.STRING);

    repositoryURL = new FileAttribute(this, "repositoryURL");
    keywords = new StringAttribute(this, "keywords");

    _attachText("_iconDescription",
                "<svg>\n"
                + "<rect x=\"0\" y=\"0\" "
                + "width=\"80\" height=\"40\" "
                + "style=\"fill:white\"/>\n"
                + "</svg>\n");
  } // end of constructor

  ///////////////////////////////////////////////////////////////////
  ////                     ports and parameters                  ////

  // The url or the path of the wsdl list/repository.
  public FileAttribute repositoryURL;
  // The keywords for the selection of web services.
  public StringAttribute keywords;
  // The ...
  public TypedIOPort result;
  public File WSmomlfile;
  ///////////////////////////////////////////////////////////////////
  ////                        public methods                     ////

  /** Callback for changes in attribute values
   *  Get the file or url selected.
   *
   *  @param a The attribute that changed.
   *  @exception IllegalActionException If the offsets array is not
   *   nondecreasing and nonnegative.
   */
  public void attributeChanged(Attribute at) throws IllegalActionException {

    if (at == repositoryURL) {
      // FIX ME: I'm no sure what to do when the user changes the wsdl
      //       list/repository. We can either clean the pre-constructed folders,
      //       or we can add new actors. The actor folder will be too big if
      //       we don't delete the actors.
    } //end of if (at == wsdlUrl)

  } // end of attributeChanged

  /** Grab the WSDL urls from the file or repository url given.
   *
   *  @exception IllegalActionException If there is no director.
   */
  public void fire() throws IllegalActionException {
    super.fire();

    try {
      String strRepositoryURL = repositoryURL.getExpression();
      if (!strRepositoryURL.trim().startsWith("http://")) { // it is a file name
        File file = repositoryURL.asFile();
        _in = new FileReader(file);
      } // end of if (!strrepositoryURL.trim().startsWith("http"))
      else {
        URL url = new URL(strRepositoryURL);
        HttpURLConnection urlconn = (HttpURLConnection) url.openConnection();
        _in = new InputStreamReader(urlconn.getInputStream());
      }
      BufferedReader br = new BufferedReader(_in);
      _momlStr = "<?xml version=\"1.0\" standalone=\"no\"?> \n" +
                 "<!DOCTYPE plot PUBLIC \"-//UC Berkeley//DTD MoML 1//EN\"\n" +
                 "    \"http://ptolemy.eecs.berkeley.edu/xml/dtd/MoML_1.dtd\">" +
                 "<entity name=\"web services\" class=\"ptolemy.moml.EntityLibrary\">" +
                 "\n  <configure>\n" +
                 "<?moml \n      <group>";
      String line = "";
      while ( (line = br.readLine()) != null) {
      	int hrefInd = line.toLowerCase().indexOf("href");
      	if (hrefInd != -1) {  // TODO: needs to be modified to take care of other urls in the line
	        line = line.substring(hrefInd);
      		int indBegin = line.toLowerCase().indexOf("http://");
	        int indEnd = line.toLowerCase().indexOf(".wsdl");
	        if (indEnd == -1) {
	        	indEnd = line.toLowerCase().indexOf("?wsdl");
	        }
	        if (! (indBegin == -1) && ! (indEnd == -1)) {
	          String wsdlUrl = line.substring(indBegin , indEnd + 5);
	          //result.broadcast(new StringToken(wsdlUrl));
	          wsdlUrl = wsdlUrl.replaceAll("%2F", "/");
	          wsdlUrl = wsdlUrl.replaceAll("%3A", ":");
	          indBegin = wsdlUrl.toLowerCase().lastIndexOf("http");
	          wsdlUrl = wsdlUrl.substring(indBegin);
	          int slashIndex = wsdlUrl.lastIndexOf('/');
	          String folderName = wsdlUrl.substring(slashIndex + 1,
	                                                wsdlUrl.length() - 5);
	          /* Pseudocode for the implementation of the harvesting after this point:
	           * 1. Get the content of the "wsdlUrl" into _wsdlStr
	           */
	          try {
	            System.out.println("Getting the content of the wsdlUrl: " + wsdlUrl);
	            URL url = new URL(wsdlUrl);
	            HttpURLConnection urlconn =
	                (HttpURLConnection) url.openConnection();
	            BufferedReader in = new BufferedReader(new InputStreamReader(
	                urlconn.
	                getInputStream()));
	            _WSDLStr = "";
	            String wsdlLine = "";
	            wsdlLine = in.readLine();
	            _WSDLStr += wsdlLine;
	            while (true) {
	              wsdlLine = in.readLine();
	              if (wsdlLine == null)
	                break;
	              _WSDLStr += '\n' + wsdlLine;
	            }
	            in.close();
	            urlconn.disconnect();
	          }
	          catch (Exception e) {
	            KernelException.stackTraceToString(e);
	            System.out.println(e + "\n");
	          }

	          /* 3. Parse wsdlStr
	           * 4. Get names of the operations
	           * 5. For each operation
	           *      6. get input-output message
	           *      7. get part names and their types
	           *      8. create an actor for each operation and
	           *         save it under actor "webservices/foldername_operation"
	           *        To do this, append "kepler/lib/webservices.xml"
	           *         Append webservices with entity,
	           *          <entity name="foldername-operation" class="org.sdm.spa.WebService">
	           *             <property name="wsdlUrl" class="ptolemy.kernel.util.StringAttribute" value="http://xml.nig.ac.jp/wsdl/Blast.wsdl"/>
	           *             <property name="methodName" class="ptolemy.kernel.util.StringAttribute" value="searchParam"/>
	           *             <property name="_icon" class="ptolemy.vergil.icon.AttributeValueIcon">
	           *               <property name="attributeName" value="wsdlUrl"/>
	           *             </property>
	           *          </entity>
	           */
	          try {
	            // Parse the WSDL into a DOM object.
	            System.out.println(_WSDLStr);
	            DocumentBuilderFactory factory =
	                DocumentBuilderFactory.newInstance();
	            factory.setValidating(false);
	            DocumentBuilder parser = factory.newDocumentBuilder();
	            InputSource in = new InputSource(new StringReader(_WSDLStr));
	            Document doc = parser.parse(in);

	            // Query the input message for the given operation (method).
	            String xPath = "//portType/operation";
	            Node contextNode = doc.getDocumentElement();
	            NodeIterator nodeIter =
	                XPathAPI.selectNodeIterator(contextNode, xPath);
	            Node node;
	            // For each node
	            _momlStr += "<entity name=\"" + folderName + "\" class=\"ptolemy.moml.EntityLibrary\">\n";
	            while ( (node = nodeIter.nextNode()) != null) {
	              _methodNameStr = ( (Element) node).getAttribute("name");
	              _momlStr += " <entity name=\"" + folderName + "_" +
	                  _methodNameStr +
	                  "\" class=\"org.sdm.spa.WebServiceStub\">\n" +
	                  "    <property name=\"wsdlUrl\" class=\"ptolemy.kernel.util.StringParameter\" value=\"" +
	                  wsdlUrl + "\">\n" +
	                  "        <property name=\"_style\" class=\"ptolemy.actor.gui.style.NotEditableLineStyle\"/>" +
	                  "\n    </property>\n" +
	                  "    <property name=\"methodName" +
	                  "\" class=\"ptolemy.kernel.util.StringParameter\" value=\"" +
	                  _methodNameStr + "\">\n" +
	                  "        <property name=\"_style\" class=\"ptolemy.actor.gui.style.NotEditableLineStyle\"/>" +
	                  "\n    </property>\n";
	              try {

	               // Query the input message for the given operation (method).
	                xPath = "//portType/operation[@name=\"" + _methodNameStr +
	                    "\"]/input";
	                contextNode = doc.getDocumentElement();
	                NodeIterator nodeIterInMess =
	                    XPathAPI.selectNodeIterator(contextNode, xPath);
	                // For each node
	                while ( (node = nodeIterInMess.nextNode()) != null) {
	                  _inMessage = ( (Element) node).getAttribute("name");
	                }

	                // Query the output message for the given operation (method).
	                xPath = "//portType/operation[@name=\"" + _methodNameStr +
	                    "\"]/output";
	                contextNode = doc.getDocumentElement();
	                NodeIterator nodeIterOutMess = XPathAPI.selectNodeIterator(contextNode, xPath);
	                // For each node
	                while ( (node = nodeIterOutMess.nextNode()) != null) {
	                  _outMessage = ( (Element) node).getAttribute("name");
	                }

	                // Query the parts of the queried input message.
	                xPath = "//message[@name=\"" + _inMessage +
	                    "\"]/part";
	                contextNode = doc.getDocumentElement();
	                NodeIterator nodeIterInput = XPathAPI.selectNodeIterator(contextNode, xPath);
	                String partName = "";
	                String typeStr = "";
	                // For each node
	                while ( (node = nodeIterInput.nextNode()) != null) {
	                  partName = ( (Element) node).getAttribute("name");
	                  typeStr = ( (Element) node).getAttribute("type");
	                  _momlStr += "         <port name=\"" + partName +
	                     "\" class=\"ptolemy.actor.TypedIOPort\">\n" +
	                     "             <property name=\"input\"/>\n" +
	                     "             <property name=\"type\" class=\"ptolemy.actor.TypeAttribute\" value=\"string\"/>\n" +
	                     "         </port>\n";

	                }

	                // Query the parts of the queried outputput message.
	                xPath = "//message[@name=\"" + _outMessage +
	                    "\"]/part";
	                contextNode = doc.getDocumentElement();
	                NodeIterator nodeIterOutput = XPathAPI.selectNodeIterator(contextNode, xPath);
	                partName = "";
	                typeStr = "";
	                // For each node
	                while ( (node = nodeIterOutput.nextNode()) != null) {
	                  partName = ( (Element) node).getAttribute("name");
	                  typeStr = ( (Element) node).getAttribute("type");
	                  _momlStr += "         <port name=\"" + partName +
	                      "\" class=\"ptolemy.actor.TypedIOPort\">\n" +
	                      "             <property name=\"output\"/>\n" +
	                      "             <property name=\"type\" class=\"ptolemy.actor.TypeAttribute\" value=\"string\"/>\n" +
	                      "         </port>\n";
	                }
	                _momlStr += " </entity>\n";
	              }
	              catch (Exception e) {
	                System.out.println(e);
	              }
	            }
	            _momlStr += " </entity>\n";
	            /* 9. Dynamically update folders with the new webservices.xml file.
	             *
	             * WSExecuter is a class that executes a specified web service.
	                 * This class will be used to execute the web services in the harvester.
	             *
	             */
	          }
	          catch (Exception e) {
	            System.out.println(e);
	          }

	          //System.out.println(_momlStr);
	        }
      	}
      }
      _momlStr += "</group>\n?>\n</configure>\n</entity>\n";
      result.broadcast(new StringToken(_momlStr));
      _in.close();
      br.close();

      WSmomlfile = new File(MOMLFILEPATH);
      if (WSmomlfile == null) {
           throw new IllegalArgumentException("File should not be null.");
         }
         if (!WSmomlfile.exists()) {
           throw new FileNotFoundException ("File does not exist: " + WSmomlfile);
         }
         if (!WSmomlfile.isFile()) {
           throw new IllegalArgumentException("Should not be a directory: " + WSmomlfile);
         }
         if (!WSmomlfile.canWrite()) {
           throw new IllegalArgumentException("File cannot be written: " + WSmomlfile);
         }

         //declared here only to make visible to finally clause; generic reference
         Writer output = null;
         try {
           //use buffering
           output = new BufferedWriter( new FileWriter(WSmomlfile) );
           output.write( _momlStr );
         }
         finally {
           //flush and close both "output" and its underlying FileWriter
           if (output != null) output.close();
         }


    }
    catch (Exception ex) {
      MessageHandler.error(
          "Error opening/updating one of the input or parameter files: ", ex);
    }
  } // end of fire

  /** There's nothing to initialize currently.
   *  @exception IllegalActionException If the parent class throws it.
   */
  public void initialize() throws IllegalActionException {
    super.initialize();
    _WSDLStr = "";
    _momlStr = "";
  } // end of initialize

  /** Post fire the actor. Return false to indicate that the
   * process has finished. If it returns true, the process will
   * continue indefinitely.
   */
  public boolean postfire() {
    return false;
  } // end of postfire

  /** Pre fire the actor.
   *  Calls the super class's prefire in case something is set there.
   */
  public boolean prefire() throws IllegalActionException {
    return super.prefire();
  } // end of prefire

  //////////////////////////////////////////////////////////////////////
  ////                         private variables                    ////

  // The WSDL string
  String _WSDLStr = "";
  // The name of the method that this web service actor binds to
  String _methodNameStr = new String();
  // Input and output messages of the web service
  public String _inMessage;
  public String _outMessage;
  // MOML addition for each web service.
  String _momlStr = "";
  // The content of the file or url selected.

  private Reader _in;

  // private String KEPLER = System.getProperty("KEPLER");
  // private String MOMLFILEPATH = KEPLER + "/configs/ptolemy/configs/spa/webservices.xml";
  private String KEPLER = System.getProperty("user.dir");
  private String MOMLFILEPATH = KEPLER + "/configs/ptolemy/configs/spa/webservices.xml";

}