/**
 * Copyright (c) 2004 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

package org.sdm.spa;

import util.UUIDGen;

import ptolemy.actor.TypedAtomicActor;
import ptolemy.data.RecordToken;
import ptolemy.data.StringToken;
import ptolemy.data.IntToken;
import ptolemy.data.type.RecordType;
import ptolemy.actor.TypedIOPort;
import ptolemy.data.DoubleToken;
import ptolemy.data.LongToken;
import ptolemy.data.BooleanToken;
import ptolemy.data.Token;
import ptolemy.data.expr.FileParameter;
import ptolemy.data.expr.Parameter;
import ptolemy.data.type.BaseType;
import ptolemy.data.type.Type;
import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.util.*;

import util.XSLT;
//For java APIs
import java.io.*;
import java.util.*;
//For XSLT APIs
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.stream.StreamSource;
import javax.xml.transform.stream.StreamResult;

//////////////////////////////////////////////////////////////////////////
//// XSLTTransformer
/**
 * Given an XSL transformation file, performs the specified transformation on
 * the input XML doc.
 *
   @author Ilkay Altintas
   @version $Id: XSLTActor.java,v 1.14 2004/08/26 00:47:34 altintas Exp $
   @category.name data transformation
 */

public class XSLTActor
    extends TypedAtomicActor {

  public FileParameter xslFileName;
  public TypedIOPort xmlIn;
  public TypedIOPort htmlOut;

  public String htmlStr = "";

  public XSLTActor(CompositeEntity container, String name) throws
      NameDuplicationException, IllegalActionException {

    super(container, name);

    xmlIn = new TypedIOPort(this, "xmlIn", true, false);
    htmlOut = new TypedIOPort(this, "htmlOut", false, true);

    xmlIn.setTypeEquals(BaseType.STRING);
    htmlOut.setTypeEquals(BaseType.STRING);
    xslFileName = new FileParameter(this, "XSLT File Path");

    new Attribute(xmlIn, "_showName");
    new Attribute(htmlOut, "_showName");

    _attachText("_iconDescription",
                    "<svg>\n"
                    + "<rect x=\"0\" y=\"0\" "
                    + "width=\"60\" height=\"30\" "
                    + "style=\"fill:white\"/>\n"
                    + "<text x=\"10\" y=\"25\""
                    + "style=\"font-size:16; fill:blue; font-family:SansSerif\">"
                    + "XSLT</text>\n"
                    + "</svg>\n");

  }

  public void fire() throws IllegalActionException {
    super.fire();

    String xmlStr = ( (StringToken) xmlIn.get(0)).stringValue();
//    System.out.println(xmlStr);
    //FIX ME: Apply XSLT transform to xmlStr and generate htmlStr...
    //htmlStr = xmlStr;

      //generate xml file name for storing the xml string.
      UUIDGen uuidgen = new UUIDGen();
      String fileName = uuidgen.generateUUID();
      System.out.println(fileName + ".xml");
      /* FIX ME: Need to delete the file if it is generated but an
               exception occurs before the end.
       */

      File xmlFile = new File(fileName + ".xml");

      try {
      BufferedWriter out = new BufferedWriter(new FileWriter(xmlFile.getPath()));
      out.write(xmlStr);
      out.close();
    } catch(Exception e) {
//      System.out.println("file out error");
    }
    File xslFile = xslFileName.asFile();

    XSLT XSLTransformer = new XSLT();
    htmlStr = XSLTransformer.Transform(xmlFile,xslFile);
    sendToken(htmlStr);
    xmlFile.delete();
  }

  protected void sendToken(Object o) throws ClassCastException,
      IllegalActionException {
    htmlOut.send(0, new StringToken( (String) o));
    //getToken will always return a stringToken here.
  }

  public boolean prefire() throws IllegalActionException {
    return super.prefire();
  }

  public boolean postfire() {
    htmlStr = "";
    return true;
  }

}
